
#include "AssignReferenceStatement.h"

AssignReferenceStatement::AssignReferenceStatement (const string &_sourceReferenceName, const string &_destinationReferenceName) {
  sourceReference = NULL;
  destinationReference = NULL;
  sourceReferenceName = _sourceReferenceName;
  destinationReferenceName = _destinationReferenceName;
}

AssignReferenceStatement::~AssignReferenceStatement (void) {
}

void AssignReferenceStatement::exec (void) {
  PRINT1("\nAsignacion del objeto indicado por la referencia '" + sourceReferenceName + "' a la referencia '" + destinationReferenceName + "'.");
  sourceReference = thread->findReference (sourceReferenceName);
  // Busqueda de la referencia a 'String' para futuros usos.
  Reference* stringReference = findString();
  // Si no se encuentra el objeto 'String' entonces se finaliza.
  // Este es un error grave.
  if (stringReference == NULL) {
    return;
  }
  if (sourceReference == NULL) {
    // Si el objeto es un 'StringObject' entonces se crea en el contexto actual.
    if (Basic::isNitroString (sourceReferenceName)) {
      PRINT1("\n  '" + sourceReferenceName + "' no encontrada (referencia origen). Como es de tipo string entonces se crea.");
      // Creacion del objeto con sus correspondientes miembros 'id' y 'super'.
      StringObject* object = new StringObject (sourceReferenceName);
      sourceReference = new Reference (object);
      sourceReference->getObject()->insertMember (ID, new Reference (object));
      sourceReference->getObject()->insertMember (SUPER, new Reference (stringReference->getObject()));
      thread->getLastContext()->insertReference (sourceReferenceName, new Reference (object));
    }
    // Si el objeto no es de tipo 'StringObject' entonces se finaliza.
    else {
      PRINT1("\n  '" + sourceReferenceName + "' no encontrada (referencia objeto).");
      return;
    }
    return;
  }
  destinationReference = thread->findReference (destinationReferenceName);
  if (destinationReference == NULL) {
    PRINT1("\n  '" + destinationReferenceName + "' no encontrada (referencia destino).");
    PRINT1("\n  Creacion de la nueva referencia: " + destinationReferenceName + ".");
    destinationReference = new Reference (sourceReference->getObject());
    thread->getLastContext()->insertReference (destinationReferenceName, destinationReference);
    return;
  }
  destinationReference->setObject (sourceReference->getObject());
}

