
#include "AsynchronousMemberEvaluationStatement.h"
#include "PrimitiveStringObject.h"
#include "ThreadPool.h"
#include "Evaluation.h"

AsynchronousMemberEvaluationStatement::AsynchronousMemberEvaluationStatement (const string &_objectReferenceName, const string &_memberReferenceName, const ParamsList &_paramsList): MemberEvaluationStatement (_objectReferenceName, _memberReferenceName, _paramsList) {
}

AsynchronousMemberEvaluationStatement::~AsynchronousMemberEvaluationStatement (void) {
}

void AsynchronousMemberEvaluationStatement::exec (void) {
  PRINT1("\nEvaluacion asincrona del miembro '" + memberReferenceName + "' de la referencia '" + objectReferenceName + "'.");
  // Inicializacion.
  memberReference = NULL;
  objectReference = NULL;
  string bufferString = "";
  // Busqueda de la referencia a 'String' para futuros usos.
  Reference* stringReference = findString();
  // Si no se encuentra el objeto 'String' entonces se finaliza.
  // Este es un error grave.
  if (stringReference == NULL) {
    return;
  }
  // Busqueda de la referencia al objeto cuyo miembro se desea ejecutar.
  objectReference = thread->findReference (objectReferenceName);
  // Si el objeto no existe...
  if (objectReference == NULL) {
    // Si el objeto es un 'StringObject' entonces se crea en el contexto actual.
    if (Basic::isNitroString (objectReferenceName)) {
      PRINT1("\n  '" + objectReferenceName + "' no encontrada (referencia objeto). Como es de tipo string entonces se crea.");
      // Creacion del objeto con sus correspondientes miembros 'id' y 'super'.
      StringObject* object = new StringObject (objectReferenceName);
      objectReference = new Reference (object);
      objectReference->getObject()->insertMember (ID, new Reference (object));
      objectReference->getObject()->insertMember (SUPER, new Reference (stringReference->getObject()));
      thread->getLastContext()->insertReference (objectReferenceName, new Reference (object));
    }
    // Si el objeto no es de tipo 'StringObject' entonces se finaliza.
    else {
      PRINT1("\n  '" + objectReferenceName + "' no encontrada (referencia objeto).");
      return;
    }
  }
  // Busqueda del miembro al que se accede.
  memberReference = objectReference->getObject()->findMember (memberReferenceName);
  if (memberReference == NULL) {
    PRINT1("\n  '" + memberReferenceName + "' no encontrada (referencia miembro).");
    return;
  }
  // Busqueda del objeto propietario del miembro que se va a evaluar. Puede ser el
  // propio objeto o alguno de sus padres.
  memberOwnerReference = objectReference->getObject()->findMemberOwner (memberReferenceName);
  if (memberOwnerReference == NULL) {
    PRINT1("\n  '" + memberReferenceName + "' no encontrada (referencia miembro).");
    return;
  }
  // Evaluacion del miembro, en funcion de si corresponde a un objeto primitivo o no.
  PrimitiveStringObject* pso = dynamic_cast<PrimitiveStringObject*>(memberReference->getObject());
  if (pso != NULL) {
    PRINT1("\n  Es un objeto primitivo.");
    pso->evaluate (thread, objectReferenceName, memberReferenceName, "", paramsList);
  }
  else {
    // Se comprueba si el objeto posee el miembro ':'.
    Reference* twoPointsReference = objectReference->getObject()->findLocalMember ("<:>");
    if ((twoPointsReference != NULL) && (!(objectReference->getObject()->isTwoPointsLocked()))) {
      PRINT1("\n  El objeto posee el miembro ':'.");
      twoPointsMethod (twoPointsReference, NULL, paramsList, false, false);
      return;
    }
    else {
      // Busqueda de la referencia a 'nil' para futuros usos.
      Reference* nilReference = thread->findReference ("nil");
      // Si no se encuentra el objeto 'nil' entonces se finaliza.
      // Este es un error grave.
      if (nilReference == NULL) {
        PRINT1("\n  Referencia 'nil' no encontrada.");
        return;
      }
      // Se comprueba si el objeto miembro posee el miembro ')('.
      Reference* asynchronousBracketsReference = memberReference->getObject()->findLocalMember ("<)(>");
      if (asynchronousBracketsReference == NULL) {
        // Comprobar que el objeto miembro es un "StringObject"??
        StringObject* so = dynamic_cast<StringObject*>(memberReference->getObject());
        if (so != NULL) {
          PRINT1("\n  Es un objeto string.");
          // Creacion de una evaluacion para evaluar el objeto string.
          char *p = new char [so->getValue().length() + 1];
          strcpy (p, Basic::nitroStringToString (so->getValue()).c_str());
	        p [so->getValue().length()] = '\0';
          istrstream* isString = new istrstream (p);
          ThreadPool::threadPool.setCurrentEvaluation (ThreadPool::threadPool.newEvaluation (new istream (isString->rdbuf()), thread));
        }
        else {
          PRINT1("\n  No es un objeto string.");
          return;
        }
      }
      else {
        PRINT1("\n  El objeto miembro posee el metodo ')('.");
        // Comprobar que el objeto '()' es un "StringObject"??
        StringObject* so = dynamic_cast<StringObject*>(asynchronousBracketsReference->getObject());
        if (so != NULL) {
          PRINT1("\n  ')(' es un objeto string.");
          // Creacion de una evaluacion para evaluar el objeto string.
          char *p = new char [so->getValue().length() + 1];
          strcpy (p, Basic::nitroStringToString (so->getValue()).c_str());
	        p [so->getValue().length()] = '\0';
          istrstream* isString = new istrstream (p);
          ThreadPool::threadPool.setCurrentEvaluation (ThreadPool::threadPool.newEvaluation (new istream (isString->rdbuf()), thread));
        }
        else {
          PRINT1("\n  '()' no es un objeto string.");
          return;
        }
      }
      // Creacion del objeto "params" con los miembros adecuados.
      PRINT1("\n  Creacion del objeto 'params' y de sus referencias.");
      ParamsList::const_iterator start, stop;
      start = paramsList.begin();
      stop = paramsList.end();
      StringObject* params = new StringObject (PARAMS);
      // Creacion de los miembros 'id' y 'super'.
      params->insertMember (ID, new Reference (params));
      params->insertMember (SUPER, new Reference (stringReference->getObject()));
      // Creacion de todos los miembros necesarios para gestionar los parametros.
      // Los nombres de los objetos miembro seran desde 0 hasta el numero de parametros
      // menos 1.
      int i = 0;  // Controla el nombre de los objetos miembro del objeto 'params'.
      bool exit = false;  // Variable para controlar la ejecucion de un bucle.
      for ( ; (start != stop) && (!exit); ++start) {
        // Creacion del nombre del objeto miembro.
        bufferString = "<" + Basic::intToString (i) + ">";
        // Busqueda del objeto correspondiente al parametro en creacion y creacion
        // de la referencia correspondiente.
        PRINT1("\n    Creacion de la referencia al parametro '" + bufferString + "'.");
        Reference* referenceAux = thread->findReference (*start);
        // Si no se ha encontrado el objeto para crear el miembro de 'params' se
        // intenta crear un nuevo objeto de tipo StringObject.
        if (exit = (referenceAux == NULL)) {
          PRINT1("\n    Referencia al parametro '" + bufferString + "' no encontrada. Se crea el nuevo objeto.");
          if (Basic::isNitroString (*start)) {
            PRINT1("\n    Creacion de la referencia al parametro '" + bufferString + "' (parametro '" + (*start) + "').");
//**            StringObject* so2 = new StringObject (Basic::nitroStringToString (*start));
            StringObject* so2 = new StringObject (*start);
            so2->insertMember (ID, new Reference (so2));
            so2->insertMember (SUPER, new Reference (stringReference->getObject()));
            referenceAux = new Reference (so2);
            params->insertMember (bufferString, referenceAux);
            thread->getLastContext()->insertReference ((*start), new Reference (so2));
            PRINT1("\n    Referencia al parametro '" + bufferString + "' insertada.");
            exit = false;
          }
          else {
            PRINT1("\n    Imposible crear el objeto referenciado por el parametro '" + bufferString + "'.");
          }
        }
        else {
          // Se inserta el nuevo parametro como miembro del objeto 'params'.
          params->insertMember (bufferString, new Reference (referenceAux->getObject()));
          PRINT1("\n    Referencia al parametro '" + bufferString + "' encontrada e insertada.");
        }
        i++;
      }
      if (exit)
        return;
      // Si se esta ejecutando un miembro ')(' se debe aniadir un parametro mas.
      if (asynchronousBracketsReference != NULL) {
        bufferString = "<" + Basic::intToString (i) + ">";
        params->insertMember (bufferString, new Reference (memberReference->getObject()));
      }
      // Creacion de la referencia "sender" dentro del nuevo contexto.
      PRINT1("\n  Creacion de la referencia 'sender' dentro del nuevo contexto.");
      thread->getLastContext()->insertReference (SENDER, new Reference (objectReference->getObject()));
      // Creacion de la referencia "mySelf" dentro del nuevo contexto.
      PRINT1("\n  Creacion de la referencia 'mySelf' dentro del nuevo contexto.");
      thread->getLastContext()->insertReference (MYSELF, new Reference (memberOwnerReference->getObject()));
      // Creacion de la referencia "params" dentro del nuevo contexto.
      PRINT1("\n  Creacion de la referencia 'params' dentro del nuevo contexto.");
      thread->getLastContext()->insertReference (PARAMS, new Reference (params));
    }
  }
}
