
#include "AsynchronousReferenceEvaluationStatement.h"
#include "ThreadPool.h"

AsynchronousReferenceEvaluationStatement::AsynchronousReferenceEvaluationStatement (const string &_objectReferenceName, const ParamsList &_paramsList): ReferenceEvaluationStatement (_objectReferenceName, _paramsList) {
}

AsynchronousReferenceEvaluationStatement::~AsynchronousReferenceEvaluationStatement (void) {
}

void AsynchronousReferenceEvaluationStatement::exec (void) {
  PRINT1("\nEvaluacion asincrona de la referencia '" + objectReferenceName + "'.");
  // Busqueda de la referencia a 'String' para futuros usos.
  Reference* stringReference = thread->findReference ("String");
  // Si no se encuentra el objeto 'String' entonces se finaliza.
  // Este es un error grave.
  if (stringReference == NULL) {
    PRINT1("\n  Referencia 'String' no encontrada.");
    return;
  }
  // Busqueda de la referencia a 'nil' para futuros usos.
  Reference* nilReference = thread->findReference ("nil");
  // Si no se encuentra el objeto 'nil' entonces se finaliza.
  // Este es un error grave.
  if (nilReference == NULL) {
    PRINT1("\n  Referencia 'nil' no encontrada.");
    return;
  }
  // Busqueda de la referencia.
  objectReference = thread->findReference (objectReferenceName);
  // Si no se encuentra entonces se finaliza.
  if (objectReference == NULL) {
    PRINT1("\n  Referencia 'nil' no encontrada.");
    return;
  }
  // Se comprueba que el objeto referenciado es un objectString.
  StringObject* so = dynamic_cast<StringObject*>(objectReference->getObject());
  if (so != NULL) {
    // Creacion de una subevaluacion para evaluar el objeto string.
    string code = Basic::nitroStringToString (so->getValue());
    char *p = new char [code.length() + 1];
	  strcpy (p, code.c_str());
	  p [code.length()] = '\0';
    istrstream* isString = new istrstream (p);
    ThreadPool::threadPool.setCurrentEvaluation (ThreadPool::threadPool.newEvaluation (new istream (isString->rdbuf()), thread));
    // Creacion del objeto "params" con los miembros adecuados.
    PRINT1("\n  Creacion del objeto 'params' y de sus referencias.");
    ParamsList::const_iterator start, stop;
    start = paramsList.begin();
    stop = paramsList.end();
    StringObject* params = new StringObject (PARAMS);
    // Creacion de los miembros 'id' y 'super'.
    params->insertMember (ID, new Reference (params));
    params->insertMember (SUPER, new Reference (stringReference->getObject()));
    // Creacion de todos los miembros necesarios para gestionar los parametros.
    // Los nombres de los objetos miembro seran desde 0 hasta el numero de parametros
    // menos 1.
    int i = 0;  // Controla el nombre de los objetos miembro del objeto 'params'.
    bool exit = false;  // Variable para controlar la ejecucion de un bucle.
    for ( ; (start != stop) && (!exit); ++start) {
      // Creacion del nombre del objeto miembro.
      string bufferString = "<" + Basic::intToString (i) + ">";
      // Busqueda del objeto correspondiente al parametro en creacion y creacion
      // de la referencia correspondiente.
      PRINT1("\n    Creacion de la referencia al parametro '" + bufferString + "'.");
      Reference* referenceAux = thread->findReference (*start);
      // Si no se ha encontrado el objeto para crear el miembro de 'params' se
      // intenta crear un nuevo objeto de tipo StringObject.
      if (exit = (referenceAux == NULL)) {
        PRINT1("\n    Referencia al parametro '" + bufferString + "' no encontrada. Se crea el nuevo objeto.");
        if (Basic::isNitroString (*start)) {
          PRINT1("\n    Creacion de la referencia al parametro '" + bufferString + "' (parametro '" + (*start) + "').");
//**          StringObject* so2 = new StringObject (Basic::nitroStringToString (*start));
          StringObject* so2 = new StringObject (*start);
          so->insertMember (ID, new Reference (so2));
          so->insertMember (SUPER, new Reference (stringReference->getObject()));
          referenceAux = new Reference (so2);
          params->insertMember (bufferString, referenceAux);
          thread->getLastContext()->insertReference ((*start), new Reference (so2));
          PRINT1("\n    Referencia al parametro '" + bufferString + "' insertada.");
          exit = false;
        }
        else {
          PRINT1("\n    Imposible crear el objeto referenciado por el parametro '" + bufferString + "'.");
        }
      }
      else {
        // Se inserta el nuevo parametro como miembro del objeto 'params'.
        params->insertMember (bufferString, new Reference (referenceAux->getObject()));
        PRINT1("\n    Referencia al parametro '" + bufferString + "' encontrada e insertada.");
      }
      i++;
    }
    if (exit)
      return;
    // Creacion de la referencia "params" dentro del nuevo contexto.
    PRINT1("\n  Creacion de la referencia 'params' dentro del nuevo contexto.");
    thread->getLastContext()->insertReference (PARAMS, new Reference (params));
  }
  else
    PRINT1("\n  La referencia NO apunta a un objeto string.");
}
