
#include "Evaluation.h"
#include "ThreadPool.h"

Evaluation::Evaluation (istream* is, Context* baseContext) {
  finished = false;
  thread = new Thread();
  subEvaluation = NULL;
  parser = new Parser (is);
  setBaseContext (baseContext);
}

Evaluation::Evaluation (istream* is, Thread* _thread) {
  finished = false;
  thread = _thread;
  subEvaluation = NULL;
  parser = new Parser (is);
}

Evaluation::~Evaluation (void) {
  delete parser;
}

bool Evaluation::isFinished (void) const {
  return finished;
}

void Evaluation::setFinished (const bool _finished) {
  finished = _finished;
}

Thread* Evaluation::getThread (void) const {
  return thread;
}

void Evaluation::setThread (Thread* _thread) {
  thread = _thread;
}

Evaluation* Evaluation::getSubEvaluation (void) const {
  return subEvaluation;
}

void Evaluation::setSubEvaluation (Evaluation* _subEvaluation) {
  subEvaluation = _subEvaluation;
}

Evaluation* Evaluation::newSubEvaluation (const string &in) {
	char *p = new char [in.length() + 1];
  strcpy (p, Basic::nitroStringToString(in).c_str());
	p [in.length()] = '\0';
  istrstream* isString = new istrstream (p);
  subEvaluation = new Evaluation (new istream (isString->rdbuf()), thread);
  // Creacion del nuevo contexto de ejecucion.
  PRINT1 ("\n  Creacion del nuevo contexto.");
  thread->newContext();
  return subEvaluation;
}

void Evaluation::evaluate (void) {
  ThreadPool::threadPool.setCurrentEvaluation (this);
  if (subEvaluation == NULL) {
    Statement* statement = parser->nextStatement();
    if (statement == NULL) {
      finished = true;
    }
    else {
      statement->setThread (thread);
      statement->exec();
      delete statement;
    }
  }
  else {
    if (subEvaluation->isFinished()) {
      // Eliminacion del contexto utilizado por la sentencia para su ejecucion.
      thread->deleteLastContext();
      delete subEvaluation;
      subEvaluation = NULL;
    }
    else {
      subEvaluation->evaluate();
    }
  }
  ThreadPool::threadPool.setCurrentEvaluation (NULL);
}

void Evaluation::setBaseContext (Context* baseContext) {
  thread->insertContext (baseContext);
}
