
#include "HasPrimitiveStringObject.h"

HasPrimitiveStringObject::HasPrimitiveStringObject (const string &_value): PrimitiveStringObject (_value) {
}

HasPrimitiveStringObject::~HasPrimitiveStringObject (void) {
}

void HasPrimitiveStringObject::evaluate (Thread* thread, const string &objectReferenceName, const string &memberReferenceName, const string &destinationReferenceName, const ParamsList paramsList) {
  PRINT1 ("\n  Ejecucion del miembro 'has'.");
  // Busqueda de la referencia a 'nil' para futuros usos.
  Reference* nilReference = thread->findReference ("nil");
  if (nilReference == NULL) {
    PRINT1 ("\n    Referencia 'nil' no encontrada.");
    return;
  }
  // Busqueda de la referencia a 'nil' para futuros usos.
  Reference* stringReference = thread->findReference ("String");
  if (stringReference == NULL) {
    PRINT1 ("\n    Referencia 'String' no encontrada.");
    return;
  }
  // Busqueda de la referencia al objeto 'return'.
  Reference* destinationReference = NULL;
  if (destinationReferenceName == "") {
    PRINT1 ("\n    No se ha indicado la referencia de destino.");
    return;
  }
  else {
    destinationReference = thread->findReference (destinationReferenceName);
    if (destinationReference == NULL) {
      PRINT1 ("\n    '" + destinationReferenceName + "' no encontrada (referencia destino).");
      PRINT1 ("\n    Creacion de la nueva referencia: " + destinationReferenceName + " (apunta a 'nil').");
      destinationReference = new Reference (nilReference->getObject());
      thread->getLastContext()->insertReference (destinationReferenceName, destinationReference);
      PRINT1 ("\n    Referencia '" + destinationReferenceName + "' creada correctamente.");
    }
  }
  // Busqueda de la referencia al objeto en el que se buscara el miembro.
  Reference* objectReference = thread->findReference (objectReferenceName);
  if (objectReference == NULL) {
    PRINT1 ("\n    No se ha encontrado la referencia al objeto 'sender'.");
    return;
  }
  // Busqueda de la referencia al objeto que indica la clave a buscar.
  if (paramsList.size() == 1) {
    // Analisis del parametro 0.
    Reference* param0Reference = thread->findReference (paramsList.front());
    if (param0Reference == NULL) {
      PRINT1 ("\n    No se ha encontrado la referencia correspondiente al parametro 0 (" + paramsList.front() + "). Se intenta crear una nueva con el objeto correspondiente.");
      if (Basic::isNitroString (paramsList.front())) {
        PRINT1 ("\n    El parametro 0 es un 'NitroString' por lo que se crea un nuevo objeto y una nueva referencia al mismo.");
        StringObject* stringObjectParam0 = new StringObject (paramsList.front());
        stringObjectParam0->insertMember (ID, new Reference (stringObjectParam0));
        stringObjectParam0->insertMember (SUPER, new Reference (stringReference->getObject()));
        param0Reference = new Reference (stringObjectParam0);
      }
      else {
        PRINT1 ("\n    No se ha encontrado el parametro 0 (" + paramsList.front() + ") y no es un 'NitroString'.");
        return;
      }
    }
    StringObject* so = dynamic_cast<StringObject*>(param0Reference->getObject()->getId()->getObject());
    if (so != NULL) {
      if (objectReference->getObject()->findLocalMember (so->getValue()) != NULL) {
        PRINT1 ("\n    El objeto posee el miembro consultado.");
        // Busqueda de la referencia a 'true'.
        Reference* trueReference = thread->findReference ("true");
        if (trueReference == NULL) {
          PRINT1 ("\n    Referencia 'true' no encontrada.");
          return;
        }
        destinationReference->setObject (trueReference->getObject());
        return;
      }
      else {
        PRINT1 ("\n    El objeto no posee el miembro consultado.");
        // Busqueda de la referencia a 'false'.
        Reference* falseReference = thread->findReference ("false");
        if (falseReference == NULL) {
          PRINT1 ("\n    Referencia 'false' no encontrada.");
          return;
        }
        destinationReference->setObject (falseReference->getObject());
        return;
      }
    }
    else {
      PRINT1 ("\n    No se ha podido acceder al identificador del parametro 0.");
      return;
    }
  }
  else {
    PRINT1 ("\n    No se ha indicado un unico parametro.");
    return;
  }
}
