
#include "Lexer.h"

Lexer::Lexer (istream* _data) {
  data = _data;
  line = 1;
  nextToken();
}

Lexer::~Lexer (void) {
}

char Lexer::readChar (void) {
  char car;
  if (buffer.length() > 0) {
    car = buffer[buffer.length() - 1];
    buffer = buffer.substr (0, buffer.length() - 1);
  }
  else {
    data->get (car);
    if (data->eof())
      car = EOF;
  }
  return car;
}

void Lexer::returnChar (char c) {
  buffer = buffer + c;
}

void Lexer::nextToken (void) {
  string lexema;
  char c = readChar();
  char c2, c3;
  while ((c == ' ') || (c == '\t') || (c == '\n') || (c == '\\')) {
    if (c == '\n')
      line++;
    if (c == '\\') {
      if ((c2 = readChar()) == '*') {
        lexema = c;      
        lexema = lexema + c2;
        while (((c2 != '\\') || (c != '*')) && (c != EOF)) {
          c2 = c;
          c = readChar();
          if (c == '\n')
            line++;
          lexema = lexema + c;
        }
        if (c == EOF)
          token = TK_END;
        else
          token = TK_COMMENT;
        return;
      }
      else {
        token = TK_ESCAPE;
        returnChar (c2);
        return;
      }
    }
    c = readChar();
  }
  if (c == EOF) {
    token = TK_END;
    return;
  }
  if (c == '(') {
    token = TK_LPAREN;
    return;
  }
  if (c == '.') {
    token = TK_POINT;
    return;
  }
  if (c == ')') {
    token = TK_RPAREN;
    return;
  }
  if (c == '<') {
    c2 = readChar();
    if (c2 == '-') {
      c3 = readChar();
      if (c3 == '>') {
        token = TK_STRING;
        result = "<->";
        return;
      }
      else {
        returnChar (c3);
        token = TK_ASSIGN_REF;
        return;
      }
    }
    else {
      returnChar (c2);
      lexema = c;
      int count = 1;
      do {
        c2 = c;
        c = readChar();        
        switch (c) {
          case '%': lexema = lexema + '%';
                    break;
          case '\n': lexema = lexema + c;
                     line++;
                     break;
          case '-': lexema = lexema + c;
                    if (c2 == '<')
                      count--;
                    break;
          case '<': lexema = lexema + c;
                    count++;
                    break;
          case '>': lexema = lexema + c;
                    if (c2 != '-')
                      count--;
                    break;
          case '\\': c2 = c;
                     c = readChar();
                     switch (c) {
                       case 'n': lexema = lexema + '\n';
                                 break;
                       case '-': lexema = lexema + '-';
                                 break;
                       case 't': lexema = lexema + '\t';
                                 break;
                       case '\\': lexema = lexema + '\\';
                                  break;
                       case '>': lexema = lexema + '\\' + '>';
                                 break;
                       case '<': lexema = lexema + '\\' + '<';
                                 break;
                     }
                     break;
          default: lexema = lexema + c;
        }
      } while ((count > 0) && (c != EOF));
      if (c == EOF) {
        token = TK_END;
      }
      else {
        result = lexema;
        token = TK_STRING;
      }
      return;
    }
  }
  if (c == '>') {
    token = TK_RANGLE;
    return;
  }
  if (c == '-') {
    c2 = readChar();
    if (c2 == '>')
      token = TK_NEW_REF;
    else {
      returnChar (c2);
      token = TK_MINUS;
    }
    return;
  }
  if (c == ',') {
    token = TK_COMMA;
    return;
  }
  if (c == ':') {
    token = TK_TWO_POINTS;
    return;
  }
  if (c == ';') {
    token = TK_SEMICOLON;
    return;
  }
  if (c == '=') {
    c2 = readChar();
    if (c2 == '=') {
      token = TK_EQUAL;
      return;
    }
  }
  if (c == '+') {
    token = TK_ADD;
    return;
  }
  if (c == '*') {
    token = TK_MUL;
    return;
  }
  if (c == '%') {
    token = TK_MOD;
    return;
  }
  if (c == '/') {
    token = TK_DIV;
    return;
  }
  if (isdigit (c)) {
    lexema = c;
    while (isdigit (c = readChar())) {
      lexema = lexema + c;
    }
    returnChar (c);
    result = lexema;
    token = TK_INTEGER;
    return;
  }
  if ((isalpha (c)) || (c == '_') || (c == '%')) {
    lexema = c;
    while (isalpha (c = readChar()) || isdigit (c) || (c == '_'))
      lexema = lexema + c;
    returnChar (c);
    result = lexema;
    token = TK_IDENTIFIER;
    return;
  }
  PRINT2("\nLexical error (line: %d): Token desconocido.", getLine());
}

int Lexer::getToken (void) const {
  return token;
}

string Lexer::getResult (void) const {
  return result;
}

long Lexer::getLine (void) const {
  return line;
}

