#include "Machine.h"
#include <fstream>
#include <sstream>
#include <string>
#include <iostream>
#include <string>
#include <string.h>
#include "ThreadPool.h"
#include "Debugger.h"
#include "Evaluation.h"

using namespace std;

void runVM() {
    ThreadPool::threadPool.eraseFinishedEvaluations();

    for (int i = 0; i < ThreadPool::threadPool.evaluationsSize(); i++) {
        if (!ThreadPool::threadPool.getThreadOfEvaluation(i)->isLocked()) {
          if (!ThreadPool::threadPool.isEvaluationFinished(i)) {
            Debugger::debugger.print ("Nueva evaluacion.");
            ThreadPool::threadPool.evaluateEvaluation(i);
          }
          else  ThreadPool::threadPool.insertFinishedEvaluation(i);
        }
      }
}

// * Virtual Machine Interface Statements

void newReference (const string &reference) {
  // * Crea una referencia en el contexto 0
  string statement = "-> " + reference + ";";
  char *p = new char [statement.length() + 1];
  strcpy (p, statement.c_str());
  p [statement.length()] = '\0';
  istringstream* isString = new istringstream(p);
  Debugger::debugger.print ("NEW EVALUATION: " + statement);
  ThreadPool::threadPool.newEvaluation (isString);
  runVM();
}

string getMember (const string &object, const string &member, const string &result) {
  // * Crea una nueva instruccin de acceso al miembro y espera a que
  //   termine (cuando la haya evaluado el hilo principal "threadLoop")
  string statement = result + " <- " + object + ":" + member + ";";
  bool finish = false;
  char *p = new char [statement.length() + 1];
  strcpy (p, statement.c_str());
  p [statement.length()] = '\0';
  istringstream* isString = new istringstream (p);
  Evaluation* evaluation = NULL;
  Debugger::debugger.print ("NEW EVALUATION: " + statement);
  evaluation = ThreadPool::threadPool.newEvaluation (isString);

  runVM();

  //** Si no se ha creado antes la referencia destino entonces no devuelve el resultado.
  Reference* resultReference = evaluation->getThread()->findReference(result);
  string res;
  if (resultReference != NULL) {
     StringObject* so = dynamic_cast<StringObject *>(resultReference->getObject()->getId()->getObject());
     if (so == NULL) res = "ERROR 2";
     else res = so->getValue();
     }
 else Debugger::debugger.print ("Se ha producido un error en la espera por la liberacion del mutex (funcion getMember).");
 return res;
}

string evaluateMemberSync (const string &object, const string &member, const string &params, const string &result) {
  // * Crea una nueva instruccin de evaluacin de un miembro y espera a que
  //   termine (cuando la haya evaluado el hilo principal "threadLoop")
  string statement = object + ":" + member + "(" + params + ");";
  if (result != "")
    statement = result + " <- " + statement;
  char *p = new char [statement.length() + 1];
  strcpy (p, statement.c_str());
  p [statement.length()] = '\0';
  istringstream* isString = new istringstream (p);
  Evaluation* evaluation = NULL;
  Debugger::debugger.print ("NEW EVALUATION: " + statement);
  evaluation = ThreadPool::threadPool.newEvaluation (isString);

  runVM();


  // Se espera por la evaluacion completa de la nueva sentencia y asi poder devolver el resultado.
  string res = "ERROR";
  Reference* resultReference = evaluation->getThread()->findReference(result);
  if (resultReference != NULL) {
   StringObject* so = dynamic_cast<StringObject *>(resultReference->getObject()->getId()->getObject());
   if (so == NULL) res = "ERROR 2";
   else res = so->getValue();
   }
  else  Debugger::debugger.print ("Se ha producido un error en la espera por la liberacion del mutex (funcion getMember).");
  return res;
}

void evaluateMemberAsync (const string &object, const string &member, const string &params) {
  // * Crea una nueva instruccin de evaluacin de un miembro y
  //   finaliza sin esperar resultado alguno
  // * Crea una nueva instruccin de evaluacin de un miembro y espera a que
  //   termine (cuando la haya evaluado el hilo principal "threadLoop")
  string statement = object + ":" + member + ")" + params + "(;";
  bool finish = false;
  char *p = new char [statement.length() + 1];
  strcpy (p, statement.c_str());
  p [statement.length()] = '\0';
  istringstream* isString = new istringstream (p);
  Evaluation* evaluation = NULL;
  Debugger::debugger.print ("NEW EVALUATION: " + statement);
  evaluation = ThreadPool::threadPool.newEvaluation (isString);
  finish = (evaluation == NULL);
}

string evaluateStringSync (const string &object, const string &params, const string &result) {
  // * Crea una nueva instruccin de evaluacin de un string y espera a que
  //   termine (cuando la haya evaluado el hilo principal "threadLoop")
  string statement = object + "(" + params + ");";
  if (result != "")
    statement = result + " <- " + statement;
  bool finish = false;
  char *p = new char [statement.length() + 1];
  strcpy (p, statement.c_str());
  p [statement.length()] = '\0';
  istringstream* isString = new istringstream (p);
  Evaluation* evaluation = NULL;
  Debugger::debugger.print ("NEW EVALUATION: " + statement);
  evaluation = ThreadPool::threadPool.newEvaluation (isString);

  runVM();

  // Se espera por la evaluacion completa de la nueva sentencia y asi poder devolver el resultado.
  string res = "ERROR";
  //** Si no se ha creado antes la referencia destino entonces no devuelve el resultado.
  Reference* resultReference = evaluation->getThread()->findReference(result);
  if (resultReference != NULL) {
     StringObject* so = dynamic_cast<StringObject *>(resultReference->getObject()->getId()->getObject());
     if (so == NULL) res = "ERROR 2";
     else  res = so->getValue();
     }
  else  {
        Debugger::debugger.print ("Se ha producido un error en la espera por la liberacion del mutex (funcion getMember).");
        finish = true;
       }
  return res;
}

void evaluateStringAsync (const string &object, const string &params) {
  // * Crea una nueva instruccin de evaluacin de un string y
  //   finaliza sin esperar resultado alguno
  string statement = object + ")" + params + "(;";
  bool finish = false;
  char *p = new char [statement.length() + 1];
  strcpy (p, statement.c_str());
  p [statement.length()] = '\0';
  istringstream* isString = new istringstream (p);
  Evaluation* evaluation = NULL;
  Debugger::debugger.print ("NEW EVALUATION: " + statement);
  evaluation = ThreadPool::threadPool.newEvaluation (isString);
  finish = (evaluation == NULL);
}
