
#include "MemberAccessStatement.h"

MemberAccessStatement::MemberAccessStatement (const string &_objectReferenceName, const string &_memberReferenceName, const string &_destinationReferenceName): TwoPointsStatement (_objectReferenceName, _memberReferenceName) {
  destinationReference = NULL;
  destinationReferenceName = _destinationReferenceName;
}

MemberAccessStatement::~MemberAccessStatement (void) {
}

void MemberAccessStatement::exec (void) {
  PRINT1 ("\nAsignacion del objeto indicado por el miembro '" + memberReferenceName + "' del objeto indicado por la referencia '" + objectReferenceName + "' a la referencia '" + destinationReferenceName + "'.");
  // Busqueda de la referencia al objeto a cuyo miembro se accede.
  objectReference = thread->findReference (objectReferenceName);
  // Si el objeto no existe...
  if (objectReference == NULL) {
    // Si el objeto es un 'StringObject' entonces se crea en el contexto actual.
    if (Basic::isNitroString (objectReferenceName)) {
      PRINT1 ("\n  '" + objectReferenceName + "' no encontrada (referencia objeto). Como es de tipo string entonces se crea.");
      // Busqueda del objeto 'String'.
      Reference* stringReference = thread->getLastContext()->findReference ("String");
      // Si no se encuentra el objeto 'String' entonces se finaliza.
      // Este es un error grave.
      if (stringReference == NULL) {
        PRINT1 ("\n  Referencia 'String' no encontrada.");
        return;
      }
      // Creacion del objeto con sus correspondientes miembros 'id' y 'super'.
      StringObject* object = new StringObject (objectReferenceName);
      objectReference = new Reference (object);
      objectReference->getObject()->insertMember (ID, new Reference (object));
      objectReference->getObject()->insertMember (SUPER, new Reference (stringReference->getObject()));
      thread->getLastContext()->insertReference (objectReferenceName, new Reference (object));
    }
    // Si el objeto no es de tipo 'StringObject' entonces se finaliza.
    else {
      PRINT1 ("\n  '" + objectReferenceName + "' no encontrada (referencia objeto).");
      return;
    }
  }
  // Busqueda del miembro al que se accede.
  memberReference = objectReference->getObject()->findMember (memberReferenceName);
  if (memberReference == NULL) {
    PRINT1 ("\n  '" + memberReferenceName + "' no encontrada (referencia miembro).");
    return;
  }
  // Busqueda del objeto propietario del miembro que se va a evaluar. Puede ser el
  // propio objeto o alguno de sus padres.
  memberOwnerReference = objectReference->getObject()->findMemberOwner (memberReferenceName);
  if (memberOwnerReference == NULL) {
    PRINT1 ("\n  '" + memberReferenceName + "' no encontrada (referencia miembro).");
    return;
  }
  // Busqueda de la referencia del objeto destino de la asignacion.
  destinationReference = thread->findReference (destinationReferenceName);
  // Si la referencia del objeto destino no existe se crea en el contexto actual.
  if (destinationReference == NULL) {
    PRINT1 ("\n  '" + destinationReferenceName + "' no encontrada (referencia destino).");
    PRINT1 ("\n  Creacion de la nueva referencia: " + destinationReferenceName + ".");
    // Busqueda de la referencia a 'nil' para futuros usos.
    Reference* nilReference = thread->findReference ("nil");
    if (nilReference == NULL) {
      PRINT1 ("\n    Referencia 'nil' no encontrada.");
      return;
    }
    destinationReference = new Reference (nilReference->getObject());
    thread->getLastContext()->insertReference (destinationReferenceName, destinationReference);
  }
  // Se comprueba si el objeto posee el miembro ':'.
  Reference* twoPointsReference = objectReference->getObject()->findLocalMember ("<:>");
  if ((twoPointsReference != NULL) && (!(objectReference->getObject()->isTwoPointsLocked()))) {
    PRINT1 ("\n  El objeto posee el miembro ':'.");
    ParamsList paramsList;
    twoPointsMethod (twoPointsReference, destinationReference, paramsList, true, false);
    return;
  }
  // Asignacion propiamente dicha, si el objeto no posee el miembro ':'.
  destinationReference->setObject (memberReference->getObject());
}
