
#include "NewPrimitiveStringObject.h"

NewPrimitiveStringObject::NewPrimitiveStringObject (const string &_value): PrimitiveStringObject (_value) {
}

NewPrimitiveStringObject::~NewPrimitiveStringObject (void) {
}

void NewPrimitiveStringObject::evaluate (Thread* thread, const string &objectReferenceName, const string &memberReferenceName, const string &destinationReferenceName, const ParamsList paramsList) {
  PRINT1 ("\n  Ejecucion del miembro 'new'.");
  // Busqueda de la referencia a 'String' para futuros usos.
  Reference* stringReference = thread->findReference ("String");
  if (stringReference == NULL) {
    PRINT1 ("\n    Referencia 'String' no encontrada.");
    return;
  }
  // Busqueda de la referencia a 'nil' para futuros usos.
  Reference* nilReference = thread->findReference ("nil");
  if (nilReference == NULL) {
    PRINT1 ("\n    Referencia 'nil' no encontrada.");
    return;
  }
  // Busqueda de la referencia destino.
  // Si el resultado de la evaluacion se va a asignar a una referencia.
  Reference* destinationReference = NULL;
  if (destinationReferenceName != "") {
    destinationReference = thread->findReference (destinationReferenceName);
    if (destinationReference == NULL) {
      PRINT1 ("\n    '" + destinationReferenceName + "' no encontrada (referencia destino).");
      PRINT1 ("\n    Creacion de la nueva referencia: " + destinationReferenceName + " (apunta a 'nil').");
      destinationReference = new Reference (nilReference->getObject());
      thread->getLastContext()->insertReference (destinationReferenceName, destinationReference);
      PRINT1 ("\n    Referencia '" + destinationReferenceName + "' creada correctamente.");
    }
  }
  else {
    PRINT1 ("\n    Se necesita una referencia de destino.");
    return;
  }
  // Busqueda de la referencia al objeto cuyo miembro se desea ejecutar.
  Reference* objectReference = thread->findReference (objectReferenceName);
  // Si el objeto no existe...
  if (objectReference == NULL) {
    // Si el objeto es un 'StringObject' entonces se crea en el contexto actual.
    if (Basic::isNitroString (objectReferenceName)) {
      PRINT1 ("\n    '" + objectReferenceName + "' no encontrada (referencia objeto). Como es de tipo string entonces se crea.");
      // Creacion del objeto con sus correspondientes miembros 'id' y 'super'.
      StringObject* object = new StringObject (objectReferenceName);
      objectReference = new Reference (object);
      objectReference->getObject()->insertMember (ID, new Reference (object));
      objectReference->getObject()->insertMember (SUPER, new Reference (stringReference->getObject()));
      thread->getLastContext()->insertReference (objectReferenceName, new Reference (object));
    }
    // Si el objeto no es de tipo 'StringObject' entonces se finaliza.
    else {
      PRINT1 ("\n    '" + objectReferenceName + "' no encontrada (referencia objeto).");
      return;
    }
  }
  // Creacion del StringObject para el identificador del nuevo objeto.
  StringObject* idNewObject = new StringObject (Basic::getNewGUID());
  PRINT1 ("\n    Identificador unico generado para el nuevo objeto: " + idNewObject->getValue() + ".");
  idNewObject->insertMember (ID, new Reference (idNewObject));
  idNewObject->insertMember (SUPER, new Reference (stringReference->getObject()));
  // Creacion del nuevo objeto.
  Object* newObject = new Object (idNewObject->getValue());
  newObject->insertMember (ID, new Reference (idNewObject));
  newObject->insertMember (SUPER, new Reference (nilReference->getObject()));
  // Se devuelve el nuevo objeto referenciado por la referencia de destino.
  destinationReference->setObject (newObject);
}
