
#include "NextKeyPrimitiveStringObject.h"

NextKeyPrimitiveStringObject::NextKeyPrimitiveStringObject (const string &_value): PrimitiveStringObject (_value) {
}

NextKeyPrimitiveStringObject::~NextKeyPrimitiveStringObject (void) {
}

void NextKeyPrimitiveStringObject::evaluate (Thread* thread, const string &objectReferenceName, const string &memberReferenceName, const string &destinationReferenceName, const ParamsList paramsList) {
  PRINT1 ("\n  Ejecucion del miembro 'nextKey'.");
  // Busqueda de la referencia a 'nil' para futuros usos.
  Reference* nilReference = thread->findReference ("nil");
  if (nilReference == NULL) {
    PRINT1 ("\n    Referencia 'nil' no encontrada.");
    return;
  }
  // Busqueda de la referencia a 'nil' para futuros usos.
  Reference* stringReference = thread->findReference ("String");
  if (stringReference == NULL) {
    PRINT1 ("\n    Referencia 'String' no encontrada.");
    return;
  }
  // Busqueda de la referencia al objeto 'return'.
  Reference* destinationReference = NULL;
  if (destinationReferenceName == "") {
    PRINT1 ("\n    No se ha indicado la referencia de destino.");
    return;
  }
  else {
    destinationReference = thread->findReference (destinationReferenceName);
    if (destinationReference == NULL) {
      PRINT1 ("\n    '" + destinationReferenceName + "' no encontrada (referencia destino).");
      PRINT1 ("\n    Creacion de la nueva referencia: " + destinationReferenceName + " (apunta a 'nil').");
      destinationReference = new Reference (nilReference->getObject());
      thread->getLastContext()->insertReference (destinationReferenceName, destinationReference);
      PRINT1 ("\n    Referencia '" + destinationReferenceName + "' creada correctamente.");
    }
  }
  // Busqueda de la referencia al objeto en el que se buscara el miembro.
  Reference* objectReference = thread->findReference (objectReferenceName);
  if (objectReference == NULL) {
    PRINT1 ("\n    No se ha encontrado la referencia al objeto 'sender'.");
    return;
  }
  // Busqueda de la referencia al objeto que indica la clave a buscar.
  if (paramsList.size() == 1) {
    // Analisis del parametro 0.
    if (!Basic::isNitroString (paramsList.front())) {
      Reference* param0Reference = thread->findReference (paramsList.front());
      if (param0Reference != NULL) {
        if (objectReference->getObject()->findLocalMember (param0Reference) != NULL) {
          PRINT1 ("\n    Encontrada la referencia sobre la cual buscar la siguiente clave (" + paramsList.front() + ").");
          Reference* memberReference = objectReference->getObject()->nextKey (param0Reference);
          if (memberReference == NULL) {
            PRINT1 ("\n    No se ha encontrado la referencia buscada.");
            destinationReference->setObject (nilReference->getObject());
            return;
          }
          else {
            StringObject* so2 = dynamic_cast<StringObject*>(memberReference->getObject()->getId()->getObject());
            if (so2 != NULL) {
              PRINT1 ("\n    Encontrada la referencia buscada (" + so2->getValue() + ").");
            }
            destinationReference->setObject (memberReference->getObject());
            return;
          }
        }
        else {
          PRINT1 ("\n    No se ha encontrado la referencia a partir de la cual buscar.");
          destinationReference->setObject (nilReference->getObject());
          return;
        }
      }
      else {
        PRINT1 ("\n    No se ha podido acceder a la referencia sobre la cual buscar la siguiente clave (" + paramsList.front() + ").");
        destinationReference->setObject (nilReference->getObject());
        return;
      }
    }
    else {
      if (objectReference->getObject()->findLocalMember (paramsList.front()) != NULL) {
        PRINT1 ("\n    Encontrada la referencia sobre la cual buscar la siguiente clave (" + paramsList.front() + ").");
        Reference* memberReference = objectReference->getObject()->nextKey (paramsList.front());
        if (memberReference == NULL) {
          PRINT1 ("\n    No se ha encontrado la referencia buscada.");
          destinationReference->setObject (nilReference->getObject());
          return;
        }
        else {
          StringObject* so2 = dynamic_cast<StringObject*>(memberReference->getObject()->getId()->getObject());
          if (so2 != NULL) {
            PRINT1 ("\n    Encontrada la referencia buscada (" + so2->getValue() + ").");
          }
          destinationReference->setObject (memberReference->getObject());
          return;
        }
      }
      else {
        PRINT1 ("\n    No se ha encontrado la referencia a partir de la cual buscar.");
        destinationReference->setObject (nilReference->getObject());
        return;
      }
    }
  }
  else {
    PRINT1 ("\n    No se ha indicado un unico parametro.");
    destinationReference->setObject (nilReference->getObject());
    return;
  }
}

/*
  // Busqueda de la referencia al objeto que indica la clave a buscar.
  if (paramsList.size() == 1) {
    // Analisis del parametro 0.
    Reference* param0Reference = thread->findReference (paramsList.front());
    if (param0Reference == NULL) {
      PRINT1 ("\n    No se ha encontrado la referencia correspondiente al parametro 0 (" + paramsList.front() + "). Se intenta crear una nueva con el objeto correspondiente.");
      if (Basic::isNitroString (paramsList.front())) {
        PRINT1 ("\n    El parametro 0 es un 'NitroString' por lo que se crea un nuevo objeto y una nueva referencia al mismo.");
        StringObject* stringObjectParam0 = new StringObject (paramsList.front());
        stringObjectParam0->insertMember (ID, new Reference (stringObjectParam0));
        stringObjectParam0->insertMember (SUPER, new Reference (stringReference->getObject()));
        param0Reference = new Reference (stringObjectParam0);
      }
      else {
        PRINT1 ("\n    No se ha encontrado el parametro 0 (" + paramsList.front() + ") y no es un 'NitroString'.");
        return;
      }
    }
    StringObject* so = dynamic_cast<StringObject*>(param0Reference->getObject()->getId()->getObject());
    if (so != NULL) {
      PRINT1 ("\n    Encontrada la referencia sobre la cual buscar la siguiente clave (" + paramsList.front() + " - ID: " + so->getValue() + ").");
      Reference* memberReference = objectReference->getObject()->nextKey (so->getValue());
      if (memberReference == NULL) {
        destinationReference->setObject (nilReference->getObject());
        return;
      }
      else {
        StringObject* so2 = dynamic_cast<StringObject*>(memberReference->getObject()->getId()->getObject());
        if (so2 != NULL) {
          PRINT1 ("\n    Encontrada la referencia buscada (" + so2->getValue() + ").");
        }
        destinationReference->setObject (memberReference->getObject());
        return;
      }
    }
    else {
      PRINT1 ("\n    No se ha podido acceder al identificador del parametro 0.");
      destinationReference->setObject (nilReference->getObject());
      return;
    }
  }
  else {
    PRINT1 ("\n    No se ha indicado un unico parametro.");
    destinationReference->setObject (nilReference->getObject());
    return;
  }
*/