
#include "Object.h"
#include "StringObject.h"
#include "Debugger.h"
#include "Thread.h"

Object Object::System(0);

Object::Object (int) {
	// Creo el objeto System, sin necesidad de que ste
	// exista con anterioridad
  refCount = 0;
}

Object::Object (const string &name) {
  refCount = 0;
  twoPointsLocked = false;
  Reference* objectsMemberReference = Object::System.findLocalMember ("<objects>");
  if (objectsMemberReference != NULL) {
    objectsMemberReference->getObject()->insertMember (name, new Reference (this, false));
    PRINT1 ("\n    Se ha insertado una nueva referencia en el objeto contenedor 'objects' del objeto 'System'.");
  }
  else {
    PRINT1 ("\n    No se ha podido insertar una referencia en el objeto contenedor 'objects' del objeto 'System'.");
  }
}

Object::~Object (void) {
  map<string, Reference*>::iterator start, stop;
  start = members.begin();
  stop = members.end();
  for ( ; start != stop; ++start) {
    delete (*start).second;
  }
  members.clear();
}

ReferencesMap &Object::getMembers (void) {
  return members;
}

bool Object::insertMember (const string &memberName, Reference* member) {
  if (members.find (memberName) != members.end())
    return false;
  members[memberName] = member;
  return true;
}

Reference* Object::getSuper (void) {
  return findLocalMember (SUPER);
}

Reference* Object::getId (void) {
  return findLocalMember (ID);
}

Reference* Object::findMember (const string &memberName) {
  Object* objectAux = this;
  do {
    map<string, Reference*>::iterator finder = objectAux->getMembers().find (memberName);
    if (finder == objectAux->getMembers().end()) {
      if (objectAux->isNil()) {
        return NULL;
      }
      else {
        objectAux = objectAux->getSuper()->getObject();
      }
    }
    else {
      return objectAux->getMembers()[memberName];
    }
  } while (true);
  return objectAux->getMembers()[memberName];
}

Reference* Object::findMemberOwner (const string &memberName) {
  Object* objectAux = this;
  do {
    map<string, Reference*>::iterator finder = objectAux->getMembers().find (memberName);
    if (finder == objectAux->getMembers().end()) {
      if (objectAux->isNil()) {
        return NULL;
      }
      else {
        objectAux = objectAux->getSuper()->getObject();
      }
    }
    else {
      return new Reference (objectAux);
    }
  } while (true);
}

bool Object::isNil (void) {
  return (getSuper() == NULL) ? false : (getSuper()->getObject() == this);
}

bool Object::isLocked (void) const {
  return locked;
}

void Object::setLocked (const bool _locked) {
  locked = _locked;
}

void Object::lock (Thread* thread) {
  thread->setLocked (true);
  threadsLocked.push (thread);
  locked = true;
}

void Object::unlock (void) {
  if (!threadsLocked.empty()) {
    Thread* thread = threadsLocked.top();
    threadsLocked.pop();
    thread->setLocked (false);
  }
  locked = false;
}

int Object::getRefCount (void) const {
  return refCount;
}

void Object::printMembers (void) const {
  ReferencesMap::const_iterator start, stop;
  start = members.begin();
  stop = members.end();
  for ( ; start != stop; ++start) {
    cout << endl << "    + " << (*start).first;
  }
}

void Object::incRefCount (void) {
  refCount++;
}

void Object::decRefCount (void) {
  refCount--;
}

bool Object::updateMember (const string &memberName, Reference* member) {
  // Si se actualiza el miembro "<id>" de un objeto hay que actualizar el contenido del objeto
  // contenedor "objects" del objeto "System".
  if (memberName == ID) {
    // 1) Recoger ID antiguo.
    StringObject* so = dynamic_cast<StringObject*>(this->getId()->getObject());
    if (so != NULL) {
      // 2) Buscar objeto de clave "ID antiguo" en el miembro contenedor "<objects>" de "System".
      Reference* objectsMemberReference = Object::System.findLocalMember ("<objects>");
      if (objectsMemberReference != NULL) {
        Reference* referenceAux = objectsMemberReference->getObject()->findLocalMember (so->getValue());
        if (referenceAux != NULL) {
          // 3) Crear una referencia temporal que apunte al objeto buscado en (2).
          Object* objectAux = referenceAux->getObject();
          // 4) Eliminar el objeto de clave "ID antiguo" en el miembro contenedor "<objects>" de "System".
          objectsMemberReference->getObject()->removeMember (so->getValue());
          // 5) Aadir una nueva referencia en el miembro contenedor "<objects>" de "System" cuya clave sea
          //    el nuevo ID y que apunte al objeto referenciado por la referencia temporal creada en (4).
          objectsMemberReference->getObject()->insertMember (memberName, new Reference (objectAux, false));
          // prueba prueba que te prueba
          //objectsMemberReference->getObject()->printMembers();
        }
      }
    }
  }
  if (members.find (memberName) != members.end())
    delete members[memberName];
  members[memberName] = member;
  return true;
}

Reference* Object::findLocalMember (const string &memberName) {
  map<string, Reference*>::iterator finder = members.find (memberName);
  return (finder != members.end()) ? members[memberName] : NULL;
}

Reference* Object::findLocalMember (Reference* _memberReference) {
  ReferencesMap::const_iterator start, stop;
  start = members.begin();
  stop = members.end();
  for ( ; start != stop; ++start) {
    Reference* refAux = (*start).second;
    if (_memberReference->getObject() == refAux->getObject())
      return (*start).second;
  }
  return NULL;
}

Reference* Object::firstKey (void) const {
  if (!members.empty()) {
    ReferencesMap::const_iterator start = members.begin();
    if (start != members.end()) {
      return (*start).second;
    }
  }
  return NULL;
}

Reference* Object::nextKey (const string &memberName) {
  map<string, Reference*>::iterator finder = members.find (memberName);
  if (finder != members.end()) {
    ++finder;
    return (finder != members.end()) ? (*finder).second : NULL;
  }
  else
    return NULL;
}

Reference* Object::nextKey (Reference* _memberReference) {
  ReferencesMap::const_iterator start, stop;
  start = members.begin();
  stop = members.end();
  for ( ; start != stop; ++start) {
    Reference* refAux = (*start).second;
    if (_memberReference->getObject() == refAux->getObject()) {
      ++start;
      return (start != members.end()) ? (*start).second : NULL;
    }
  }
  return NULL;
}

void Object::removeMember (const string &memberName) {
  map<string, Reference*>::iterator finder = members.find (memberName);
  if (finder != members.end()) {
    delete (*finder).second;
    members.erase (finder);
  }
}

void Object::lockTwoPoints (void) {
  twoPointsLocked = true;
}

void Object::unlockTwoPoints (void) {
  twoPointsLocked = false;
}

bool Object::isTwoPointsLocked (void) {
  return twoPointsLocked;
}
