
#include "Parser.h"
#include "Debugger.h"

Parser::Parser (istream* _data) {
  lexer = new Lexer (_data);
}

Parser::~Parser (void) {
  delete lexer;
}

Statement* Parser::nextStatement (void) {
  string referenceName = "";
  string identifier = "";
  string stringValue = "";
  string member = "";
  ParamsList _paramsList;
  switch (lexer->getToken()) {
    // New reference statement:  NEW_REF TK_IDENTIFIER TK_SEMICOLON
    case TK_NEW_REF: lexer->nextToken();
                     if (lexer->getToken() == TK_IDENTIFIER) {
                       referenceName = lexer->getResult();
                       lexer->nextToken();
                       if (lexer->getToken() == TK_SEMICOLON) {
                         lexer->nextToken();
                         return (new NewReferenceStatement (referenceName));
                       }
                       else {
                         return NULL;
                       }
                     }
                     else {
                       PRINT2("\nSintax error (line: %d): Se esperaba un identificador.", lexer->getLine());
                       return NULL;
                     }
                     break;
    // Assign statement:  TK_IDENTIFIER ASSIGN_REF restAssignStatement TK_SEMICOLON
    // Synchronous member evaluation statement:  TK_IDENTIFIER TWO_POINTS TK_STRING TK_LPAREN params TK_RPAREN TK_SEMICOLON
    // Or asynchronous member evaluation statement:  TK_IDENTIFIER TWO_POINTS TK_STRING TK_RPAREN params TK_LPAREN TK_SEMICOLON
    case TK_IDENTIFIER: identifier = lexer->getResult();
                        lexer->nextToken();
                        switch (lexer->getToken()) {
                          // Assign statement.
                          case TK_ASSIGN_REF: lexer->nextToken();
                                              return restAssignReference (identifier);
                                              break;
                          // Member evaluation statement.
                          case TK_TWO_POINTS: lexer->nextToken();
                                              if (lexer->getToken() == TK_STRING) {
                                                member = lexer->getResult();
                                                lexer->nextToken();
                                                // Synchronous evaluation.
                                                if (lexer->getToken() == TK_LPAREN) {
                                                  lexer->nextToken();
                                                  if (parameters (_paramsList)) {
                                                    if (lexer->getToken() == TK_RPAREN) {
                                                      lexer->nextToken();
                                                      if (lexer->getToken() == TK_SEMICOLON) {
                                                        lexer->nextToken();
                                                        return (new SynchronousMemberEvaluationStatement (identifier, member, _paramsList, ""));
                                                      }
                                                      else {
                                                        return NULL;
                                                      }
                                                    }
                                                    else {
                                                      return NULL;
                                                    }                                                
                                                  }
                                                  else {
                                                    PRINT2("\nSintax error (line: %d): Lista de parametros incorrecta.", lexer->getLine());
                                                    return NULL;
                                                  }
                                                }
                                                else {
                                                  // Asynchronous evaluation.
                                                  if (lexer->getToken() == TK_RPAREN) {
                                                    lexer->nextToken();
                                                    if (parameters (_paramsList)) {
                                                      if (lexer->getToken() == TK_LPAREN) {
                                                        lexer->nextToken();
                                                        if (lexer->getToken() == TK_SEMICOLON) {
                                                          lexer->nextToken();
                                                          return (new AsynchronousMemberEvaluationStatement (identifier, member, _paramsList));
                                                        }
                                                        else {
                                                          return NULL;
                                                        }
                                                      }
                                                      else {
                                                        PRINT2("\nSintax error (line: %d): Se esperaba '('.", lexer->getLine());
                                                        return NULL;
                                                      }                                                
                                                    }
                                                    else {
                                                      PRINT2("\nSintax error (line: %d): Lista de parametros incorrecta.", lexer->getLine());
                                                      return NULL;
                                                    }
                                                  }
                                                  else {
                                                    PRINT2("\nSintax error (line: %d): Se esperaba '(' o ')'.", lexer->getLine());
                                                    return NULL;
                                                  }
                                                }
                                              }
                                              else {
                                                PRINT2("\nSintax error (line: %d): Despues de ':' debe ir un string.", lexer->getLine());
                                                return NULL;
                                              }
                                              break;
                          case TK_LPAREN: lexer->nextToken();
                                          _paramsList.clear();
                                          if (parameters (_paramsList)) {
                                            if (lexer->getToken() == TK_RPAREN) {
                                              lexer->nextToken();
                                              if (lexer->getToken() == TK_SEMICOLON) {
                                                lexer->nextToken();
                                                return (new SynchronousReferenceEvaluationStatement (identifier, _paramsList, ""));
                                              }
                                              else {
                                                return NULL;
                                              }
                                            }
                                            else {
                                              PRINT2("\nSintax error (line: %d): Se esperaba ')'.", lexer->getLine());
                                              return NULL;
                                            }                                                
                                          }
                                          else {
                                            PRINT2("\nSintax error (line: %d): Lista de parametros incorrecta.", lexer->getLine());
                                            return NULL;
                                          }
                          case TK_RPAREN: lexer->nextToken();
                                          _paramsList.clear();
                                          if (parameters (_paramsList)) {
                                            if (lexer->getToken() == TK_LPAREN) {
                                              lexer->nextToken();
                                              if (lexer->getToken() == TK_SEMICOLON) {
                                                lexer->nextToken();
                                                return (new AsynchronousReferenceEvaluationStatement (identifier, _paramsList));
                                              }
                                              else {
                                                return NULL;
                                              }
                                            }
                                            else {
                                              PRINT2("\nSintax error (line: %d): Se esperaba ')'.", lexer->getLine());
                                              return NULL;
                                            }                                                
                                          }
                                          else {
                                            PRINT2("\nSintax error (line: %d): Lista de parametros incorrecta.", lexer->getLine());
                                            return NULL;
                                          }
                          default: PRINT2("\nSintax error (line: %d): Se esperaba '<-' o ':' (identifier: " + identifier + ").", lexer->getLine());
                                   return NULL;
                                   break;
                        }
                        break;
    // Synchronous member evaluation statement:  TK_STRING TWO_POINTS TK_STRING TK_LPAREN params TK_RPAREN TK_SEMICOLON
    // Asynchronous member evaluation statement:  TK_STRING TWO_POINTS TK_STRING TK_RPAREN params TK_LPAREN TK_SEMICOLON
    // Synchronous TK_STRING evaluation statement:  TK_STRING TWO_POINTS TK_STRING TK_LPAREN params TK_RPAREN TK_SEMICOLON
    // Or asynchronous TK_STRING evaluation statement:  TK_STRING TWO_POINTS TK_STRING TK_RPAREN params TK_LPAREN TK_SEMICOLON
    case TK_STRING: stringValue = lexer->getResult();
                    lexer->nextToken();
                    switch (lexer->getToken()) {
                      // Synchronous evaluation.
                      case TK_LPAREN: lexer->nextToken();
                                      if (parameters (_paramsList)) {
                                        if (lexer->getToken() == TK_RPAREN) {
                                          lexer->nextToken();
                                          if (lexer->getToken() == TK_SEMICOLON) {
                                            lexer->nextToken();
                                            return (new SynchronousStringEvaluationStatement (stringValue, _paramsList, ""));
                                          }
                                          else {
                                            return NULL;
                                          }
                                        }
                                        else {
                                          PRINT2("\nSintax error (line: %d): Se esperaba ')'.", lexer->getLine());
                                          return NULL;
                                        }                                                                          
                                      }
                                      else {
                                        PRINT2("\nSintax error (line: %d): Lista de parametros incorrecta.", lexer->getLine());
                                        return NULL;
                                      }
                                      break;
                      // Asynchronous evaluation.
                      case TK_RPAREN: lexer->nextToken();
                                      if (parameters (_paramsList)) {
                                        if (lexer->getToken() == TK_LPAREN) {
                                          lexer->nextToken();
                                          if (lexer->getToken() == TK_SEMICOLON) {
                                            lexer->nextToken();
                                            return (new AsynchronousStringEvaluationStatement (stringValue, _paramsList));
                                          }
                                          else {
                                            return NULL;
                                          }
                                        }
                                        else {
                                          PRINT2("\nSintax error (line: %d): Se esperaba '('.", lexer->getLine());
                                          return NULL;
                                        }                                                                          
                                      }
                                      else {
                                        PRINT2("\nSintax error (line: %d): Lista de parametros incorrecta.", lexer->getLine());
                                        return NULL;
                                      }
                                      break;
                      // Member evaluation statement.
                      case TK_TWO_POINTS: lexer->nextToken();
                                          if (lexer->getToken() == TK_STRING) {
                                            member = lexer->getResult();
                                            lexer->nextToken();
                                            // Synchronous evaluation.
                                            if (lexer->getToken() == TK_LPAREN) {
                                              lexer->nextToken();
                                              if (parameters (_paramsList)) {
                                                if (lexer->getToken() == TK_RPAREN) {
                                                  lexer->nextToken();
                                                  if (lexer->getToken() == TK_SEMICOLON) {
                                                    lexer->nextToken();
                                                    return (new SynchronousMemberEvaluationStatement (stringValue, member, _paramsList, ""));
                                                  }
                                                  else {
                                                    return NULL;
                                                  }
                                                }
                                                else {
                                                  PRINT2("\nSintax error (line: %d): Se esperaba ')'.", lexer->getLine());
                                                  return NULL;
                                                }                                            
                                              }
                                              else {
                                                PRINT2("\nSintax error (line: %d): Lista de parametros incorrecta.", lexer->getLine());
                                                return NULL;
                                              }
                                            }
                                            else {
                                              if (lexer->getToken() == TK_RPAREN) {
                                                lexer->nextToken();
                                                if (parameters (_paramsList)) {
                                                  if (lexer->getToken() == TK_LPAREN) {
                                                    lexer->nextToken();
                                                    if (lexer->getToken() == TK_SEMICOLON) {
                                                      lexer->nextToken();
                                                      return (new AsynchronousMemberEvaluationStatement (stringValue, member, _paramsList));
                                                    }
                                                    else {
                                                      return NULL;
                                                    }
                                                  }
                                                  else {
                                                    PRINT2("\nSintax error (line: %d): Se esperaba ')'.", lexer->getLine());
                                                    return NULL;
                                                  }                                            
                                                }
                                                else {
                                                  PRINT2("\nSintax error (line: %d): Lista de parametros incorrecta.", lexer->getLine());
                                                  return NULL;
                                                }
                                              }
                                              else {
                                                return NULL;
                                              }
                                            }
                                          }
                                          else {
                                            PRINT2("\nSintax error (line: %d): Despues de ':' debe ir un TK_STRING.", lexer->getLine());
                                            return NULL;
                                          }
                                          break;
                      default: PRINT2("\nSintax error (line: %d): Se esperaba '(', ')' o ':'.", lexer->getLine());
                               return NULL;
                               break;
                    }
                    break;
    case TK_COMMENT: lexer->nextToken();
                     return (new CommentStatement());
                     break;
    case TK_SEMICOLON: lexer->nextToken();
                       return NULL;
                       break;
//    case TK_END: return (true);
//                 break;
    default: return NULL;
             break;
  }
}

Statement* Parser::restAssignReference (const string &destinationReferenceName) {
  string objectReferenceName = "";
  string memberReferenceName = "";
  string stringReferenceName = "";
  ParamsList _paramsList;
  switch (lexer->getToken()) {
    // Member evaluation statement or member access statement.
    case TK_IDENTIFIER: objectReferenceName = lexer->getResult();
                        lexer->nextToken();
                        switch (lexer->getToken()) {
                          // Member evaluation statement.
                          case TK_TWO_POINTS: lexer->nextToken();
                                              if (lexer->getToken() == TK_STRING) {
                                                memberReferenceName = lexer->getResult();
                                                lexer->nextToken();
                                                // Synchronous evaluation.
                                                if (lexer->getToken() == TK_LPAREN) {
                                                  lexer->nextToken();
                                                  _paramsList.clear();
                                                  if (parameters (_paramsList)) {
                                                    if (lexer->getToken() == TK_RPAREN) {
                                                      lexer->nextToken();
                                                      if (lexer->getToken() == TK_SEMICOLON) {
                                                        lexer->nextToken();
                                                        return (new SynchronousMemberEvaluationStatement (objectReferenceName, memberReferenceName, _paramsList, destinationReferenceName));
                                                      }
                                                      else {
                                                        return NULL;
                                                      }
                                                    }
                                                    else {
                                                      PRINT2("\nSintax error (line: %d): Se esperaba ')'.", lexer->getLine());
                                                      return NULL;
                                                    }                                                
                                                  }
                                                  else {
                                                    PRINT2("\nSintax error (line: %d): Lista de parametros incorrecta.", lexer->getLine());
                                                    return NULL;
                                                  }
                                                }
                                                else {
                                                  // Asynchronous evaluation (no tiene mucho sentido ya que no se
                                                  // pueden realizar asignaciones de evaluaciones asincronas).
                                                  if (lexer->getToken() == TK_RPAREN) {
                                                    lexer->nextToken();
                                                    _paramsList.clear();
                                                    if (parameters (_paramsList)) {
                                                      if (lexer->getToken() == TK_LPAREN) {
                                                        lexer->nextToken();
                                                        if (lexer->getToken() == TK_SEMICOLON) {
                                                          lexer->nextToken();
                                                          return (new AsynchronousMemberEvaluationStatement (objectReferenceName, memberReferenceName, _paramsList));
                                                        }
                                                        else {
                                                          return NULL;
                                                        }
                                                      }
                                                      else {
                                                        PRINT2("\nSintax error (line: %d): Se esperaba '('.", lexer->getLine());
                                                        return NULL;
                                                      }                                                
                                                    }
                                                    else {
                                                      PRINT2("\nSintax error (line: %d): Lista de parametros incorrecta.", lexer->getLine());
                                                      return NULL;
                                                    }
                                                  }
                                                  else {
                                                    // Member access statement
                                                    if (lexer->getToken() == TK_SEMICOLON) {
                                                      lexer->nextToken();
                                                      return (new MemberAccessStatement (objectReferenceName, memberReferenceName, destinationReferenceName));
                                                    }
                                                    else
                                                      return NULL;
                                                  }
                                                }
                                              }
                                              else {
                                                PRINT2("\nSintax error (line: %d): Despues de ':' debe ir un TK_STRING.", lexer->getLine());
                                                return NULL;
                                              }
                                              break;
                          case TK_LPAREN: lexer->nextToken();
                                          _paramsList.clear();
                                          if (parameters (_paramsList)) {
                                            if (lexer->getToken() == TK_RPAREN) {
                                              lexer->nextToken();
                                              if (lexer->getToken() == TK_SEMICOLON) {
                                                lexer->nextToken();
                                                return (new SynchronousReferenceEvaluationStatement (objectReferenceName, _paramsList, destinationReferenceName));
                                              }
                                              else {
                                                return NULL;
                                              }
                                            }
                                            else {
                                              PRINT2("\nSintax error (line: %d): Se esperaba ')'.", lexer->getLine());
                                              return NULL;
                                            }                                                
                                          }
                                          else {
                                            PRINT2("\nSintax error (line: %d): Lista de parametros incorrecta.", lexer->getLine());
                                            return NULL;
                                          }
                          case TK_RPAREN: lexer->nextToken();
                                          _paramsList.clear();
                                          if (parameters (_paramsList)) {
                                            if (lexer->getToken() == TK_LPAREN) {
                                              lexer->nextToken();
                                              if (lexer->getToken() == TK_SEMICOLON) {
                                                lexer->nextToken();
                                                return (new AsynchronousReferenceEvaluationStatement (objectReferenceName, _paramsList));
                                              }
                                              else {
                                                return NULL;
                                              }
                                            }
                                            else {
                                              PRINT2("\nSintax error (line: %d): Se esperaba ')'.", lexer->getLine());
                                              return NULL;
                                            }                                                
                                          }
                                          else {
                                            PRINT2("\nSintax error (line: %d): Lista de parametros incorrecta.", lexer->getLine());
                                            return NULL;
                                          }
                          case TK_SEMICOLON: lexer->nextToken();
                                             return (new AssignReferenceStatement (objectReferenceName, destinationReferenceName));
                                             break;
                          default: return NULL;
                                   break;
                        }
                        break;
    case TK_STRING: stringReferenceName = lexer->getResult();
                    lexer->nextToken();
                    switch (lexer->getToken()) {
                      // Member evaluation statement.
                      case TK_TWO_POINTS: lexer->nextToken();
                                          if (lexer->getToken() == TK_STRING) {
                                            memberReferenceName = lexer->getResult();
                                            lexer->nextToken();
                                            // Synchronous evaluation.
                                            if (lexer->getToken() == TK_LPAREN) {
                                              lexer->nextToken();
                                              _paramsList.clear();
                                              if (parameters (_paramsList)) {
                                                if (lexer->getToken() == TK_RPAREN) {
                                                  lexer->nextToken();
                                                  if (lexer->getToken() == TK_SEMICOLON) {
                                                    lexer->nextToken();
                                                    return (new SynchronousMemberEvaluationStatement (stringReferenceName, memberReferenceName, _paramsList, destinationReferenceName));
                                                  }
                                                  else {
                                                    return NULL;
                                                  }
                                                }
                                                else {
                                                  PRINT2("\nSintax error (line: %d): Se esperaba ')'.", lexer->getLine());
                                                  return NULL;
                                                }                                                
                                              }
                                              else {
                                                PRINT2("\nSintax error (line: %d): Lista de parametros incorrecta.", lexer->getLine());
                                                return NULL;
                                              }
                                            }
                                            else {
                                              // Asynchronous evaluation.
                                              if (lexer->getToken() == TK_RPAREN) {
                                                lexer->nextToken();
                                                _paramsList.clear();
                                                if (parameters (_paramsList)) {
                                                  if (lexer->getToken() == TK_LPAREN) {
                                                    lexer->nextToken();
                                                    if (lexer->getToken() == TK_SEMICOLON) {
                                                      lexer->nextToken();
                                                      return (new AsynchronousMemberEvaluationStatement (stringReferenceName, memberReferenceName, _paramsList));
                                                    }
                                                    else {
                                                      return NULL;
                                                    }
                                                  }
                                                  else {
                                                    PRINT2("\nSintax error (line: %d): Se esperaba ')'.", lexer->getLine());
                                                    return NULL;
                                                  }                                                
                                                }
                                                else {
                                                  PRINT2("\nSintax error (line: %d): Lista de parametros incorrecta.", lexer->getLine());
                                                  return NULL;
                                                }
                                              }
                                              else {
                                                // Member access statement
                                                if (lexer->getToken() == TK_SEMICOLON) {
                                                  lexer->nextToken();
                                                  return (new MemberAccessStatement (stringReferenceName, memberReferenceName, destinationReferenceName));
                                                }
                                                else
                                                  return NULL;
                                              }
                                            }
                                          }
                      // Synchronous TK_STRING evaluation statement.
                      case TK_LPAREN: lexer->nextToken();
                                      _paramsList.clear();
                                      if (parameters (_paramsList)) {
                                        if (lexer->getToken() == TK_RPAREN) {
                                          lexer->nextToken();
                                          if (lexer->getToken() == TK_SEMICOLON) {
                                            lexer->nextToken();
                                            return (new SynchronousStringEvaluationStatement (stringReferenceName, _paramsList, destinationReferenceName));
                                          }
                                          else {
                                            PRINT2("\nSintax error (line: %d): Se esperaba ')'.", lexer->getLine());
                                            return NULL;
                                          }                                                
                                        }
                                        else {
                                          PRINT2("\nSintax error (line: %d): Se esperaba ')'.", lexer->getLine());
                                          return NULL;
                                        }                                                
                                      }
                                      else {
                                        PRINT2("\nSintax error (line: %d): Lista de parametros incorrecta.", lexer->getLine());
                                        return NULL;
                                      }
                                      break;
                      // Asynchronous evaluation.
                      case TK_RPAREN: lexer->nextToken();
                                      _paramsList.clear();
                                      if (parameters (_paramsList)) {
                                        if (lexer->getToken() == TK_LPAREN) {
                                          lexer->nextToken();
                                          if (lexer->getToken() == TK_SEMICOLON) {
                                            lexer->nextToken();
                                            return (new AsynchronousStringEvaluationStatement (stringReferenceName, _paramsList));
                                          }
                                          else {
                                            PRINT2("\nSintax error (line: %d): Se esperaba '('.", lexer->getLine());
                                            return NULL;
                                          }                                                
                                        }
                                        else {
                                          PRINT2("\nSintax error (line: %d): Se esperaba '('.", lexer->getLine());
                                          return NULL;
                                        }                                                
                                      }
                                      else {
                                        PRINT2("\nSintax error (line: %d): Lista de parametros incorrecta.", lexer->getLine());
                                        return NULL;
                                      }
                                      break;
                      case TK_SEMICOLON: lexer->nextToken();
                                         return (new AssignReferenceStatement (stringReferenceName, destinationReferenceName));
                                         break;

                      default: return NULL;
                               break;
                    }
                    break;
    case TK_END: return NULL;
                 break;
    default: return NULL;
             break;
  }
}

bool Parser::parameter (ParamsList &_paramsList) {
  if ((lexer->getToken() == TK_IDENTIFIER) || (lexer->getToken() == TK_STRING)) {
    _paramsList.push_back (lexer->getResult());
    lexer->nextToken();
    return true;
  }
  else {
    PRINT2("\nSintax error (line: %d): Se esperaba un identificador o un TK_STRING.", lexer->getLine());
    return false;
  }
}

bool Parser::paramsList (ParamsList &_paramsList) {
  if (parameter (_paramsList)) {
    if (lexer->getToken() == TK_COMMA) {
      lexer->nextToken();
      return paramsList (_paramsList);
    }
    else {
      return true;
    }
  }
  else {
    return false;
  }  
}

bool Parser::parameters (ParamsList &_paramsList) {
  if ((lexer->getToken() == TK_IDENTIFIER) || (lexer->getToken() == TK_STRING)) {
    return paramsList (_paramsList);
  }
  else {
    return true;
  }    
}

bool Parser::isFinished (void) {
  return (lexer->getToken() == TK_END);
}

