
#include "SynchronousMemberEvaluationStatement.h"
#include "PrimitiveStringObject.h"
#include "ThreadPool.h"
#include "Evaluation.h"

SynchronousMemberEvaluationStatement::SynchronousMemberEvaluationStatement (const string &_objectReferenceName, const string &_memberReferenceName, const ParamsList &_paramsList, const string &_destinationReferenceName): MemberEvaluationStatement (_objectReferenceName, _memberReferenceName, _paramsList) {
  destinationReferenceName = _destinationReferenceName;
}

SynchronousMemberEvaluationStatement::~SynchronousMemberEvaluationStatement (void) {
}

void SynchronousMemberEvaluationStatement::exec (void) {
  PRINT1 ("\nEvaluacion sincrona del miembro '" + memberReferenceName + "' de la referencia '" + objectReferenceName + "' y asignacion a la referencia '" + destinationReferenceName + "'.");
  // Inicializacion.
  destinationReference = NULL;
  memberReference = NULL;
  objectReference = NULL;
  string bufferString = "";
  // Busqueda de la referencia a 'String' para futuros usos.
  Reference* stringReference = thread->findReference ("String");
  // Si no se encuentra el objeto 'String' entonces se finaliza.
  // Este es un error grave.
  if (stringReference == NULL) {
    PRINT1 ("\n  Referencia 'String' no encontrada.");
    return;
  }
  // Busqueda de la referencia al objeto cuyo miembro se desea ejecutar.
  objectReference = thread->findReference (objectReferenceName);
  // Si el objeto no existe...
  if (objectReference == NULL) {
    // Si el objeto es un 'StringObject' entonces se crea en el contexto actual.
    if (Basic::isNitroString (objectReferenceName)) {
      PRINT1 ("\n  '" + objectReferenceName + "' no encontrada (referencia objeto). Como es de tipo string entonces se crea.");
      // Creacion del objeto con sus correspondientes miembros 'id' y 'super'.
      StringObject* object = new StringObject (objectReferenceName);
      objectReference = new Reference (object);
      object->insertMember (ID, new Reference (object));
      object->insertMember (SUPER, new Reference (stringReference->getObject()));
      thread->getLastContext()->insertReference (objectReferenceName, new Reference (object));
    }
    // Si el objeto no es de tipo 'StringObject' entonces se finaliza.
    else {
      PRINT1 ("\n  '" + objectReferenceName + "' no encontrada (referencia objeto).");
      return;
    }
  }
  // Busqueda del miembro al que se accede.
  memberReference = objectReference->getObject()->findMember (memberReferenceName);
  if (memberReference == NULL) {
    PRINT1 ("\n  '" + memberReferenceName + "' no encontrada (referencia miembro).");
    return;
  }
  // Busqueda del objeto propietario del miembro que se va a evaluar. Puede ser el
  // propio objeto o alguno de sus padres.
  memberOwnerReference = objectReference->getObject()->findMemberOwner (memberReferenceName);
  if (memberOwnerReference == NULL) {
    PRINT1 ("\n  '" + memberReferenceName + "' no encontrada (referencia miembro).");
    return;
  }
  // Evaluacion del miembro, en funcion de si corresponde a un objeto primitivo o no.
  PrimitiveStringObject* pso = dynamic_cast<PrimitiveStringObject*>(memberReference->getObject());
  if (pso != NULL) {
    PRINT1 ("\n  Es un objeto primitivo.");
    pso->evaluate (thread, objectReferenceName, memberReferenceName, destinationReferenceName, paramsList);
  }
  else {
    // Se comprueba si el objeto posee el miembro ':'.
    Reference* twoPointsReference = objectReference->getObject()->findLocalMember ("<:>");
    if ((twoPointsReference != NULL) && (!(objectReference->getObject()->isTwoPointsLocked()))) {
      PRINT1 ("\n  El objeto posee el miembro ':'.");
      // Busqueda de la referencia destino.
      // Si el resultado de la evaluacion se va a asignar a una referencia.
      if (destinationReferenceName != "") {
        // Busqueda de la referencia a 'nil' para futuros usos.
        Reference* nilReference = thread->findReference ("nil");
        // Si no se encuentra el objeto 'nil' entonces se finaliza.
        // Este es un error grave.
        if (nilReference == NULL) {
          PRINT1 ("\n  Referencia 'nil' no encontrada.");
          return;
        }
        destinationReference = thread->findReference (destinationReferenceName);
        if (destinationReference == NULL) {
          PRINT1 ("\n  '" + destinationReferenceName + "' no encontrada (referencia destino).");
          PRINT1 ("\n  Creacion de la nueva referencia: " + destinationReferenceName + " (apunta a 'nil').");
          destinationReference = new Reference (nilReference->getObject());
          thread->getLastContext()->insertReference (destinationReferenceName, destinationReference);
          PRINT1 ("\n  Referencia '" + destinationReferenceName + "' creada correctamente.");
        }
        if (destinationReference == NULL)
          return;
      }
      twoPointsMethod (twoPointsReference, destinationReference, paramsList, false, true);
      return;
    }
    else {
      // Busqueda de la referencia a 'nil' para futuros usos.
      Reference* nilReference = thread->findReference ("nil");
      // Si no se encuentra el objeto 'nil' entonces se finaliza.
      // Este es un error grave.
      if (nilReference == NULL) {
        PRINT1 ("\n  Referencia 'nil' no encontrada.");
        return;
      }
      // Busqueda de la referencia destino.
      // Si el resultado de la evaluacion se va a asignar a una referencia.
      if (destinationReferenceName != "") {
        destinationReference = thread->findReference (destinationReferenceName);
        if (destinationReference == NULL) {
          PRINT1 ("\n  '" + destinationReferenceName + "' no encontrada (referencia destino).");
          PRINT1 ("\n  Creacion de la nueva referencia: " + destinationReferenceName + " (apunta a 'nil').");
          destinationReference = new Reference (nilReference->getObject());
          thread->getLastContext()->insertReference (destinationReferenceName, destinationReference);
          PRINT1 ("\n  Referencia '" + destinationReferenceName + "' creada correctamente.");
        }
        if (destinationReference == NULL)
          return;
      }
      // Se comprueba si el objeto miembro posee el miembro '()'.
      Reference* synchronousBracketsReference = memberReference->getObject()->findLocalMember ("<()>");
      if (synchronousBracketsReference == NULL) {
        // Comprobar que el objeto miembro es un "StringObject"??
        StringObject* so = dynamic_cast<StringObject*>(memberReference->getObject());
        if (so != NULL) {
          PRINT1 ("\n  Es un objeto string.");
          // Creacion de una subevaluacion para evaluar el objeto string.
          ThreadPool::threadPool.setCurrentEvaluation (ThreadPool::threadPool.getCurrentEvaluation()->newSubEvaluation (so->getValue()));
        }
        else {
          PRINT1 ("\n  No es un objeto string.");
          return;
        }
      }
      else {
        PRINT1 ("\n  El objeto miembro posee el metodo '()'.");
        // Comprobar que el objeto '()' es un "StringObject"??
        StringObject* so = dynamic_cast<StringObject*>(synchronousBracketsReference->getObject());
        if (so != NULL) {
          PRINT1 ("\n  '()' es un objeto string.");
          // Creacion de una subevaluacion para evaluar el objeto string.
          ThreadPool::threadPool.setCurrentEvaluation (ThreadPool::threadPool.getCurrentEvaluation()->newSubEvaluation (so->getValue()));
        }
        else {
          PRINT1 ("\n  '()' no es un objeto string.");
          return;
        }
      }
      // Creacion del objeto "params" con los miembros adecuados.
      PRINT1 ("\n  Creacion del objeto 'params' y de sus referencias.");
      ParamsList::const_iterator start, stop;
      start = paramsList.begin();
      stop = paramsList.end();
      StringObject* params = new StringObject (PARAMS);
      // Creacion de los miembros 'id' y 'super'.
      params->insertMember (ID, new Reference (params));
      params->insertMember (SUPER, new Reference (stringReference->getObject()));
      // Creacion de todos los miembros necesarios para gestionar los parametros.
      // Los nombres de los objetos miembro seran desde 0 hasta el numero de parametros
      // menos 1.
      int i = 0;  // Controla el nombre de los objetos miembro del objeto 'params'.
      bool exit = false;  // Variable para controlar la ejecucion de un bucle.
      for ( ; (start != stop) && (!exit); ++start) {
        // Creacion del nombre del objeto miembro.
        bufferString = "<" + Basic::intToString (i) + ">";
        // Busqueda del objeto correspondiente al parametro en creacion y creacion
        // de la referencia correspondiente.
        PRINT1 ("\n    Creacion de la referencia al parametro '" + bufferString + "'.");
        Reference* referenceAux = thread->findReference (*start);
        // Si no se ha encontrado el objeto para crear el miembro de 'params' se
        // intenta crear un nuevo objeto de tipo StringObject.
        if (exit = (referenceAux == NULL)) {
          PRINT1 ("\n    Referencia al parametro '" + bufferString + "' no encontrada. Se crea el nuevo objeto.");
          if (Basic::isNitroString (*start)) {
            PRINT1 ("\n    Creacion de la referencia al parametro '" + bufferString + "' (parametro '" + (*start) + "').");
            StringObject* so2 = new StringObject (*start);
//**            StringObject* so2 = new StringObject (Basic::nitroStringToString (*start));
            so2->insertMember (ID, new Reference (so2));
            so2->insertMember (SUPER, new Reference (stringReference->getObject()));
            referenceAux = new Reference (so2);
            params->insertMember (bufferString, referenceAux);
            thread->getLastContext()->insertReference ((*start), new Reference (so2));
            PRINT1 ("\n    Referencia al parametro '" + bufferString + "' insertada.");
            exit = false;
          }
          else {
            PRINT1 ("\n    Imposible crear el objeto referenciado por el parametro '" + bufferString + "'.");
          }
        }
        else {
          // Se inserta el nuevo parametro como miembro del objeto 'params'.
          params->insertMember (bufferString, new Reference (referenceAux->getObject()));
          PRINT1 ("\n    Referencia al parametro '" + bufferString + "' encontrada e insertada.");
        }
        i++;
      }
      if (exit)
        return;
      // Si se esta ejecutando un miembro '()' se debe aniadir un parametro mas.
      if (synchronousBracketsReference != NULL) {
        bufferString = "<" + Basic::intToString (i) + ">";
        params->insertMember (bufferString, new Reference (memberReference->getObject()));
      }
      // Creacion de la referencia "sender" dentro del nuevo contexto.
      PRINT1 ("\n  Creacion de la referencia 'sender' dentro del nuevo contexto.");
      thread->getLastContext()->insertReference (SENDER, new Reference (objectReference->getObject()));
      // Creacion de la referencia "mySelf" dentro del nuevo contexto.
      PRINT1 ("\n  Creacion de la referencia 'mySelf' dentro del nuevo contexto.");
      thread->getLastContext()->insertReference (MYSELF, new Reference (memberOwnerReference->getObject()));
      // Creacion de la referencia "return" dentro del nuevo contexto.
      PRINT1 ("\n  Creacion de la referencia 'return' dentro del nuevo contexto.");
      thread->getLastContext()->insertReference (RETURN, destinationReference);
      // Creacion de la referencia "params" dentro del nuevo contexto.
      PRINT1 ("\n  Creacion de la referencia 'params' dentro del nuevo contexto.");
      thread->getLastContext()->insertReference (PARAMS, new Reference (params));
    }
  }
}
