
#include "ThreadPool.h"

ThreadPool ThreadPool::threadPool;

ThreadPool::ThreadPool (void) {
  baseContext = new Context();
  evaluations = EvaluationsVector ((unsigned int) 0, (Evaluation *) NULL);
}

void ThreadPool::loadImage (const string& imageName) {
  if (fileBuffer.open (imageName.c_str(), ios::in) == NULL) {
    Debugger::debugger.print ("Error al cargar el fichero de imagen.");
  }
  else {
    istream* is = new istream (&fileBuffer);
    // Creacin de los objetos primitivos.
    Object* nil_obj = new Object ("nil");
    StringObject* nil_str_obj = new StringObject ("nil");
    Object* String_obj = new Object ("String");
    StringObject* String_str_obj = new StringObject ("String");
    Object* Extern_obj = new Object ("Extern");
    StringObject* Extern_str_obj = new StringObject ("Extern");
    //Object* System_obj = new Object();
    StringObject* System_str_obj = new StringObject ("System");
    // Creacin de las referencias a los objetos primitivos.
    Reference* nil = new Reference (nil_obj);
    Reference* nil_str = new Reference (nil_str_obj);
    Reference* String = new Reference (String_obj);
    Reference* String_str = new Reference (String_str_obj);
    Reference* Extern = new Reference (Extern_obj);
    Reference* Extern_str = new Reference (Extern_str_obj);
    //Reference* System = new Reference (System_obj);
    Reference* System = new Reference (&Object::System);
    Reference* System_str = new Reference (System_str_obj);
    // Creacin de las referencias propias de los objetos primitivos.
    Reference* nil_id = new Reference (nil_str_obj);
    Reference* nil_super = new Reference (nil_obj);
    Reference* nil_str_id = new Reference (nil_str_obj);
    Reference* nil_str_super = new Reference (String_obj);
    Reference* String_id = new Reference (String_str_obj);
    Reference* String_super = new Reference (nil_obj);
    Reference* String_str_id = new Reference (String_str_obj);
    Reference* String_str_super = new Reference (String_obj);
    Reference* Extern_id = new Reference (Extern_str_obj);
    Reference* Extern_super = new Reference (nil_obj);
    Reference* Extern_str_id = new Reference (Extern_str_obj);
    Reference* Extern_str_super = new Reference (String_obj);
    Reference* System_id = new Reference (System_str_obj);
    Reference* System_super = new Reference (nil_obj);
    Reference* System_str_id = new Reference (System_str_obj);
    Reference* System_str_super = new Reference (String_obj);
    // Insercin de las referencias en cada objeto.
    nil_obj->insertMember (ID, nil_id);
    nil_obj->insertMember (SUPER, nil_super);
    nil_str_obj->insertMember (ID, nil_str_id);
    nil_str_obj->insertMember (SUPER, nil_str_super);
    String_obj->insertMember (ID, String_id);
    String_obj->insertMember (SUPER, String_super);
    String_str_obj->insertMember (ID, String_str_id);
    String_str_obj->insertMember (SUPER, String_str_super);
    Extern_obj->insertMember (ID, Extern_id);
    Extern_obj->insertMember (SUPER, Extern_super);
    Extern_str_obj->insertMember (ID, Extern_str_id);
    Extern_str_obj->insertMember (SUPER, Extern_str_super);
    Object::System.insertMember (ID, System_id);
    Object::System.insertMember (SUPER, System_super);
    System_str_obj->insertMember (ID, System_str_id);
    System_str_obj->insertMember (SUPER, System_str_super);
    // Creacion e insercion de la referencia al miembro 'objects' del objeto 'System'.
    StringObject* objects = new StringObject ("<objects>");
    objects->insertMember (ID, new Reference (objects));
    objects->insertMember (SUPER, new Reference (String_obj));
    Reference* objectsMemberReference = new Reference (objects);
    Object::System.insertMember ("<objects>", objectsMemberReference);
    // Insercion en el objeto contenedor 'objects' de 'System' de las referencias
    // a todos los objetos creados.
    objectsMemberReference->getObject()->insertMember ("nil", new Reference (nil_obj, false));
    objectsMemberReference->getObject()->insertMember ("<nil>", new Reference (nil_str_obj, false));
    objectsMemberReference->getObject()->insertMember ("String", new Reference (String_obj, false));
    objectsMemberReference->getObject()->insertMember ("<String>", new Reference (String_str_obj, false));
    objectsMemberReference->getObject()->insertMember ("Extern", new Reference (Extern_obj, false));
    objectsMemberReference->getObject()->insertMember ("<Extern>", new Reference (Extern_str_obj, false));
    objectsMemberReference->getObject()->insertMember ("System", new Reference (&Object::System, false));
    objectsMemberReference->getObject()->insertMember ("<System>", new Reference (System_str_obj, false));
    objectsMemberReference->getObject()->insertMember ("<objects>", new Reference (objects, false));
    // Creacion e insercion de la referencia al objeto primitivo 'new' de 'nil'.
    NewPrimitiveStringObject* new_pso = new NewPrimitiveStringObject ("new");
    new_pso->insertMember (ID, new Reference (new_pso));
    new_pso->insertMember (SUPER, new Reference (String_obj));
    nil_obj->insertMember ("<new>", new Reference (new_pso));
    // Creacion e insercion de la referencia al objeto primitivo 'delete' de 'nil'.
    DeletePrimitiveStringObject* delete_pso = new DeletePrimitiveStringObject ("delete");
    delete_pso->insertMember (ID, new Reference (delete_pso));
    delete_pso->insertMember (SUPER, new Reference (String_obj));
    nil_obj->insertMember ("<delete>", new Reference (delete_pso));
    // Creacion e insercion de la referencia al objeto primitivo 'set' de 'nil'.
    SetPrimitiveStringObject* set_pso = new SetPrimitiveStringObject ("set");
    set_pso->insertMember (ID, new Reference (set_pso));
    set_pso->insertMember (SUPER, new Reference (String_obj));
    nil_obj->insertMember ("<set>", new Reference (set_pso));
    // Creacion e insercion de la referencia al objeto primitivo 'has' de 'nil'.
    HasPrimitiveStringObject* has_pso = new HasPrimitiveStringObject ("has");
    has_pso->insertMember (ID, new Reference (has_pso));
    has_pso->insertMember (SUPER, new Reference (String_obj));
    nil_obj->insertMember ("<has>", new Reference (has_pso));
    // Creacion e insercion de la referencia al objeto primitivo 'firstKey' de 'nil'.
    FirstKeyPrimitiveStringObject* firstKey_pso = new FirstKeyPrimitiveStringObject ("firstKey");
    firstKey_pso->insertMember (ID, new Reference (firstKey_pso));
    firstKey_pso->insertMember (SUPER, new Reference (String_obj));
    nil_obj->insertMember ("<firstKey>", new Reference (firstKey_pso));
    // Creacion e insercion de la referencia al objeto primitivo 'nextKey' de 'nil'.
    NextKeyPrimitiveStringObject* nextKey_pso = new NextKeyPrimitiveStringObject ("nextKey");
    nextKey_pso->insertMember (ID, new Reference (nextKey_pso));
    nextKey_pso->insertMember (SUPER, new Reference (String_obj));
    nil_obj->insertMember ("<nextKey>", new Reference (nextKey_pso));
    // Creacion e insercion de la referencia al objeto primitivo 'remove' de 'nil'.
    RemovePrimitiveStringObject* remove_pso = new RemovePrimitiveStringObject ("remove");
    remove_pso->insertMember (ID, new Reference (remove_pso));
    remove_pso->insertMember (SUPER, new Reference (String_obj));
    nil_obj->insertMember ("<remove>", new Reference (remove_pso));
    // Creacion e insercion de la referencia al objeto primitivo 'get' de 'nil'.
    GetPrimitiveStringObject* get_pso = new GetPrimitiveStringObject ("get");
    get_pso->insertMember (ID, new Reference (get_pso));
    get_pso->insertMember (SUPER, new Reference (String_obj));
    nil_obj->insertMember ("<get>", new Reference (get_pso));
    // Creacion e insercion de la referencia al objeto primitivo 'getRefCount' de 'nil'.
    GetRefCountPrimitiveStringObject* getRefCount_pso = new GetRefCountPrimitiveStringObject ("getRefCount");
    getRefCount_pso->insertMember (ID, new Reference (getRefCount_pso));
    getRefCount_pso->insertMember (SUPER, new Reference (String_obj));
    nil_obj->insertMember ("<getRefCount>", new Reference (getRefCount_pso));
    // Creacion e insercion de la referencia al objeto primitivo 'enter' de 'nil'.
    EnterPrimitiveStringObject* enter_pso = new EnterPrimitiveStringObject ("enter");
    enter_pso->insertMember (ID, new Reference (enter_pso));
    enter_pso->insertMember (SUPER, new Reference (String_obj));
    nil_obj->insertMember ("<enter>", new Reference (enter_pso));
    // Creacion e insercion de la referencia al objeto primitivo 'exit' de 'nil'.
    ExitPrimitiveStringObject* exit_pso = new ExitPrimitiveStringObject ("exit");
    exit_pso->insertMember (ID, new Reference (exit_pso));
    exit_pso->insertMember (SUPER, new Reference (String_obj));
    nil_obj->insertMember ("<exit>", new Reference (exit_pso));
    // Creacion e insercion de la referencia al objeto primitivo 'invoke' de 'Extern'.
    InvokePrimitiveStringObject* invoke_pso = new InvokePrimitiveStringObject ("invoke");
    invoke_pso->insertMember (ID, new Reference (invoke_pso));
    invoke_pso->insertMember (SUPER, new Reference (String_obj));
    Extern_obj->insertMember ("<invoke>", new Reference (invoke_pso));
    // Creacion e insercion de la referencia al objeto primitivo 'lockTwoPoints' de 'nil'.
    LockTwoPointsPrimitiveStringObject* lockTwoPoints_pso = new LockTwoPointsPrimitiveStringObject ("lockTwoPoints");
    lockTwoPoints_pso->insertMember (ID, new Reference (lockTwoPoints_pso));
    lockTwoPoints_pso->insertMember (SUPER, new Reference (String_obj));
    nil_obj->insertMember ("<lockTwoPoints>", new Reference (lockTwoPoints_pso));
    // Creacion e insercion de la referencia al objeto primitivo 'unlockTwoPoints' de 'nil'.
    UnlockTwoPointsPrimitiveStringObject* unlockTwoPoints_pso = new UnlockTwoPointsPrimitiveStringObject ("unlockTwoPoints");
    unlockTwoPoints_pso->insertMember (ID, new Reference (unlockTwoPoints_pso));
    unlockTwoPoints_pso->insertMember (SUPER, new Reference (String_obj));
    nil_obj->insertMember ("<unlockTwoPoints>", new Reference (unlockTwoPoints_pso));
    // Creacion e insercion de la referencia al objeto primitivo 'print' de 'nil'.
    PrintPrimitiveStringObject* print_pso = new PrintPrimitiveStringObject ("print");
    print_pso->insertMember (ID, new Reference (print_pso));
    print_pso->insertMember (SUPER, new Reference (String_obj));
    nil_obj->insertMember ("<print>", new Reference (print_pso));
    // Creacion del contexto 0 e insercin de referencias en su lista de referencias.
    baseContext->insertReference ("nil", nil);
    baseContext->insertReference ("<nil>", nil_str);
    baseContext->insertReference ("String", String);
    baseContext->insertReference ("<String>", String_str);
    baseContext->insertReference ("Extern", Extern);
    baseContext->insertReference ("<Extern>", Extern_str);
    baseContext->insertReference ("System", System);
    baseContext->insertReference ("<System>", System_str);
//    newEvaluation (is);
    Evaluation* imageEvaluation = new Evaluation (is, baseContext);
    while (!imageEvaluation->isFinished())
      imageEvaluation->evaluate();
    delete imageEvaluation;
    currentEvaluation = NULL;
    fileBuffer.close();
  }
}

ThreadPool::~ThreadPool (void) {
}

Evaluation* ThreadPool::newEvaluation (istream* is) {
  Evaluation* evaluation = new Evaluation (is, baseContext);
  evaluations.resize ((unsigned int) (evaluations.size() + 1), evaluation);
  return evaluation;
}

Evaluation* ThreadPool::newEvaluation (istream* is, Thread* thread) {
  Evaluation* evaluation = new Evaluation (is, thread);
  evaluations.resize ((unsigned int) (evaluations.size() + 1), evaluation);
  return evaluation;
}

void ThreadPool::run (void) {
  bool end;
  do {
    end = true;
    for (int i = 0; i < evaluations.size(); i++) {
      if (!evaluations[i]->getThread()->isLocked()) {
        if (!evaluations[i]->isFinished()) {
          evaluations[i]->evaluate();
          end = false;
        }
        else {
          evaluations.erase (evaluations.begin() + i);
        }
      }
    }
  } while (!end);
}

Context* ThreadPool::getBaseContext (void) {
  return baseContext;
}

Evaluation* ThreadPool::getCurrentEvaluation (void) {
  return currentEvaluation;
}

void ThreadPool::setCurrentEvaluation (Evaluation* _currentEvaluation) {
  currentEvaluation = _currentEvaluation;
}

int ThreadPool::evaluationsSize (void) {
  return evaluations.size();
}

Thread* ThreadPool::getThreadOfEvaluation (int evaluationPosition) {
  return evaluations[evaluationPosition]->getThread();
}

bool ThreadPool::isEvaluationFinished (int evaluationPosition) {
  return evaluations[evaluationPosition]->isFinished();
}

void ThreadPool::evaluateEvaluation (int evaluationPosition) {
  Debugger::debugger.print ("Evaluando...");
  evaluations[evaluationPosition]->evaluate();
  Debugger::debugger.print ("Evaluacion terminada...");
}

void ThreadPool::eraseFinishedEvaluations (void) {
  for (int i = 0; i < finishedEvaluations.size(); i++) {
    evaluations.erase (evaluations.begin() + i);
  }
  finishedEvaluations.erase (finishedEvaluations.begin(), finishedEvaluations.end());
}

void ThreadPool::insertFinishedEvaluation (int evaluationPosition) {
  finishedEvaluations.push_back (evaluationPosition);
}

int ThreadPool::isEvaluationFinished (Evaluation* evaluation) {
  int i;
  for (i = 0; (i < evaluations.size()) && (evaluations[i] != evaluation); i++);
  if (i == evaluations.size())
    return 0;
  else
    return (evaluations[i]->isFinished()) ? 1 : 2;
}

