
#include "TwoPointsStatement.h"
#include "ThreadPool.h"
#include "Evaluation.h"

TwoPointsStatement::TwoPointsStatement (const string &_objectReferenceName, const string &_memberReferenceName) {
  objectReference = NULL;
  memberReference = NULL;
  memberOwnerReference = NULL;
  objectReferenceName = _objectReferenceName;
  memberReferenceName = _memberReferenceName;
}

TwoPointsStatement::~TwoPointsStatement (void) {
}

void TwoPointsStatement::exec (void) {
}

void TwoPointsStatement::twoPointsMethod (Reference* twoPointsMemberReference, Reference* destinationReference, ParamsList paramsList, const bool isAccess, const bool isSynchronous) {
  PRINT1 ("\nEjecucion del miembro ':'.");
  // Busqueda de la referencia a 'String' para futuros usos.
  Reference* stringReference = thread->findReference ("String");
  // Si no se encuentra el objeto 'String' entonces se finaliza.
  // Este es un error grave.
  if (stringReference == NULL) {
    PRINT1 ("\n  Referencia 'String' no encontrada.");
    return;
  }
  // Busqueda de la referencia a 'nil' para futuros usos.
  Reference* nilReference = thread->findReference ("nil");
  // Si no se encuentra el objeto 'nil' entonces se finaliza.
  // Este es un error grave.
  if (nilReference == NULL) {
    PRINT1 ("\n  Referencia 'nil' no encontrada.");
    return;
  }
  // Comprobar que el miembro ':' es un "StringObject"??
  StringObject* so = dynamic_cast<StringObject*>(twoPointsMemberReference->getObject());
  if (so != NULL) {
    PRINT1 ("\n  Es un objeto string.");
    // Creacion de una subevaluacion para evaluar el miembro ':'.
    ThreadPool::threadPool.setCurrentEvaluation (ThreadPool::threadPool.getCurrentEvaluation()->newSubEvaluation (so->getValue()));
    // Creacion del objeto "params" con los miembros adecuados.
    PRINT1 ("\n  Creacion del objeto 'params' y de sus referencias.");
    ParamsList::const_iterator start, stop;
    start = paramsList.begin();
    stop = paramsList.end();
    StringObject* params = new StringObject (PARAMS);
    // Creacion de los miembros 'id' y 'super'.
    params->insertMember (ID, new Reference (params));
    params->insertMember (SUPER, new Reference (stringReference->getObject()));
    // Creacion de todos los miembros necesarios para gestionar los parametros.
    // Los nombres de los objetos miembro seran desde 0 hasta el numero de parametros
    // menos 1.
    int i = 0;  // Controla el nombre de los objetos miembro del objeto 'params'.
    bool exit = false;  // Variable para controlar la ejecucion de un bucle.
    string bufferString;
    for ( ; (start != stop) && (!exit); ++start) {
      // Creacion del nombre del objeto miembro.
      bufferString = "<" + Basic::intToString (i) + ">";
      // Busqueda del objeto correspondiente al parametro en creacion y creacion
      // de la referencia correspondiente.
      PRINT1 ("\n    Creacion de la referencia al parametro '" + bufferString + "'.");
      Reference* referenceAux = thread->findReference (*start);
      // Si no se ha encontrado el objeto para crear el miembro de 'params' se
      // intenta crear un nuevo objeto de tipo StringObject.
      if (exit = (referenceAux == NULL)) {
        PRINT1 ("\n    Referencia al parametro '" + bufferString + "' no encontrada. Se crea el nuevo objeto.");
        if (Basic::isNitroString (*start)) {
          PRINT1 ("\n    Creacion de la referencia al parametro '" + bufferString + "' (parametro '" + (*start) + "').");
          StringObject* so2 = new StringObject (*start);
//**          StringObject* so2 = new StringObject (Basic::nitroStringToString (*start));
          so2->insertMember (ID, new Reference (so2));
          so2->insertMember (SUPER, new Reference (stringReference->getObject()));
          referenceAux = new Reference (so2);
          params->insertMember (bufferString, referenceAux);
          thread->getLastContext()->insertReference ((*start), new Reference (so2));
          PRINT1 ("\n    Referencia al parametro '" + bufferString + "' insertada.");
          exit = false;
        }
        else {
          PRINT1 ("\n    Imposible crear el objeto referenciado por el parametro '" + bufferString + "'.");
        }
      }
      else {
        // Se inserta el nuevo parametro como miembro del objeto 'params'.
        params->insertMember (bufferString, new Reference (referenceAux->getObject()));
        PRINT1 ("\n    Referencia al parametro '" + bufferString + "' encontrada e insertada.");
      }
      i++;
    }
    if (exit)
      return;
    // Insercion de los parametros propios de la invocacion al miembro ':'.
    // Busqueda de la referencia a 'true'.
    Reference* trueReference = thread->findReference ("true");
    if (trueReference == NULL) {
      PRINT1 ("\n    Referencia 'true' no encontrada.");
      return;
    }
    // Busqueda de la referencia a 'false'.
    Reference* falseReference = thread->findReference ("false");
    if (falseReference == NULL) {
      PRINT1 ("\n    Referencia 'false' no encontrada.");
      return;
    }
    // Parametro 'n' igual a 'false' si es un acceso o a 'true' si es una invocacion.
    bufferString = "<" + Basic::intToString (i) + ">";
    i++;
    if (isAccess)
      params->insertMember (bufferString, new Reference (falseReference->getObject()));
    else
      params->insertMember (bufferString, new Reference (trueReference->getObject()));
    // Parametro 'n+1' sera una referencia al nombre del miembro.
    bufferString = "<" + Basic::intToString (i) + ">";
    i++;
    params->insertMember (bufferString, new Reference (memberReference->getObject()->getId()->getObject()));
    // Parametro 'n+2' igual a 'false' si es un acceso o una invocacion asincrona
    // o igual a 'true' si es una invocacion sincrona.
    bufferString = "<" + Basic::intToString (i) + ">";
    if (isAccess || !isSynchronous)
      params->insertMember (bufferString, new Reference (falseReference->getObject()));
    else
      params->insertMember (bufferString, new Reference (trueReference->getObject()));
    // Creacion de la referencia "sender" dentro del nuevo contexto.
    PRINT1 ("\n  Creacion de la referencia 'sender' dentro del nuevo contexto.");
    thread->getLastContext()->insertReference (SENDER, new Reference (objectReference->getObject()));
    // Creacion de la referencia "mySelf" dentro del nuevo contexto.
    PRINT1 ("\n  Creacion de la referencia 'mySelf' dentro del nuevo contexto.");
    thread->getLastContext()->insertReference (MYSELF, new Reference (memberOwnerReference->getObject()));
    // Creacion de la referencia "return" dentro del nuevo contexto.
    if (isAccess || isSynchronous) {
      PRINT1 ("\n  Creacion de la referencia 'return' dentro del nuevo contexto.");
      thread->getLastContext()->insertReference (RETURN, destinationReference);
    }
    // Creacion de la referencia "params" dentro del nuevo contexto.
    PRINT1 ("\n  Creacion de la referencia 'params' dentro del nuevo contexto.");
    thread->getLastContext()->insertReference (PARAMS, new Reference (params));
  }
}
